import logging
import os
import unittest
from unittest.mock import patch

from janus.utils.logger import LogFilter, create_logger


class TestLogger(unittest.TestCase):
    def setUp(self):
        self.log_filter = LogFilter()
        self.record = logging.LogRecord(
            name="test", level=20, pathname="", lineno=0, msg="", args=(), exc_info=None
        )

    def test_log_filter(self):
        self.record.msg = "dealloc"
        self.assertFalse(self.log_filter.filter(self.record))

        self.record.msg = "That model is currently overloaded"
        self.assertFalse(self.log_filter.filter(self.record))

        self.record.msg = "Batches: "
        self.assertFalse(self.log_filter.filter(self.record))

        self.record.msg = "Using default tokenizer."
        self.assertFalse(self.log_filter.filter(self.record))

        self.record.msg = "Load pretrained SentenceTransformer"
        self.assertFalse(self.log_filter.filter(self.record))

        self.record.msg = "Use pytorch device"
        self.assertFalse(self.log_filter.filter(self.record))

        self.record.msg = "creating"
        self.assertFalse(self.log_filter.filter(self.record))

        self.record.msg = "cc -f"
        self.assertFalse(self.log_filter.filter(self.record))

        self.record.msg = "c++"
        self.assertFalse(self.log_filter.filter(self.record))

        self.record.msg = "error_code=context_length_exceeded"
        self.assertFalse(self.log_filter.filter(self.record))

        self.record.msg = "NumExpr"
        self.assertFalse(self.log_filter.filter(self.record))

        self.record.msg = "cc -shared"
        self.assertFalse(self.log_filter.filter(self.record))

        self.record.msg = "This is a test message"
        self.assertTrue(self.log_filter.filter(self.record))

    def test_create_logger(self):
        with patch.dict(os.environ, {"LOGLEVEL": "DEBUG"}):
            logger = create_logger("test")
            self.assertEqual(logger.level, logging.DEBUG)

        with patch.dict(os.environ, {}, clear=True):
            logger = create_logger("test")
            self.assertEqual(logger.level, logging.INFO)


if __name__ == "__main__":
    unittest.main()



from enum import Enum
from typing import Any, Dict, Set


class EmbeddingType(Enum):
    SOURCE = 1
    REQUIREMENT = 2
    SUMMARY = 3
    PSEUDO = 4
    TARGET = 5  # placeholder embeddings, are these useful for analysis?


CUSTOM_SPLITTERS: Set[str] = {"mumps", "binary", "ibmhlasm"}

LANGUAGES: Dict[str, Dict[str, Any]] = {
    "ada": {
        "comment": "--",
        "suffix": "adb",
        "url": "https://github.com/briot/tree-sitter-ada",
        "example": 'put_line("Hello, World!");\n',
    },
    "agda": {
        "comment": "--",
        "suffix": "agda",
        "url": "https://github.com/tree-sitter/tree-sitter-agda",
        "example": 'postulate HelloWorld : String;\nHelloWorld = "Hello, World!";\n',
    },
    "x86asm": {
        "comment": "//",
        "suffix": "s",
        "url": "https://github.com/bearcove/tree-sitter-x86asm",
        "example": "mov     rax, 60\nxor       rdi, rdi\n",
    },
    "bash": {
        "comment": "#",
        "suffix": "sh",
        "url": "https://github.com/tree-sitter/tree-sitter-bash",
        "example": 'echo "Hello world"\n',
    },
    "binary": {
        "comment": None,
        "suffix": "bin",
        "url": "https://github.com/tree-sitter/tree-sitter-c",
        "example": "04 00 00 00 cd 48 65 6c 6c 6f 2c 20 77 6f 72 6c 64",
    },
    "beancount": {
        "comment": ";",
        "suffix": "beancount",
        "url": "https://github.com/zwpaper/tree-sitter-beancount",
        "example": '2023-01-01 custom "Hello, World!"\n',
    },
    "bigquery": {
        "comment": "--",
        "suffix": "sql",
        "url": "https://github.com/takegue/tree-sitter-sql-bigquery",
        "example": 'SELECT "Hello, World!" AS message;\n',
    },
    "c": {
        "comment": "//",
        "suffix": "c",
        "url": "https://github.com/tree-sitter/tree-sitter-c",
        "example": (
            '#include <stdio.h>\n\nint main() {\n    printf("Hello, World!\\n");\n'
            "    return 0;\n}\n"
        ),
        "functional_node_types": ["function_definition"],
        "comment_node_type": "comment",
    },
    "capnp": {
        "comment": "#",
        "suffix": "capnp",
        "url": "https://github.com/amaanq/tree-sitter-capnp",
        "example": (
            "using Capnp;\n\nstruct HelloWorld @0x1234 {\n  greeting @0 :Text;\n}\n"
        ),
    },
    "cmake": {
        "comment": "#",
        "suffix": "cmake",
        "url": "https://github.com/uyha/tree-sitter-cmake",
        "example": (
            "cmake_minimum_required(VERSION 3.0)\nproject(HelloWorld)\n\n"
            'message("Hello, World!")\n'
        ),
    },
    "comment": {
        "comment": "#",
        "suffix": "comment",
        "url": "https://github.com/stsewd/tree-sitter-comment",
        "example": "# This is a comment\n",
    },
    "commonlisp": {
        "comment": ";;",
        "suffix": "lisp",
        "url": "https://github.com/theHamsta/tree-sitter-commonlisp",
        "example": '(format t "Hello, World!~%")\n',
    },
    "config": {
        "comment": "#",
        "suffix": None,
        "url": "https://github.com/metio/tree-sitter-ssh-client-config",
        "example": (
            "# Configuration for SSH Client\nHost my_server\n"
            "  HostName example.com\n  User my_username\n"
        ),
    },
    "css": {
        "comment": "/*",
        "suffix": "css",
        "url": "https://github.com/tree-sitter/tree-sitter-css",
        "example": 'body {\n    content: "Hello, World!";\n}\n',
    },
    "cuda": {
        "comment": "//",
        "suffix": "cu",
        "url": "https://github.com/theHamsta/tree-sitter-cuda",
        "example": (
            '#include <iostream>\n\n__global__ void hello() {\n    printf("Hello, '
            'World!\\n");\n}\n\nint main() {\n    hello<<<1, 1>>>();\n    '
            "cudaDeviceSynchronize();\n    return 0;\n}\n"
        ),
    },
    "d": {
        "comment": "//",
        "suffix": "d",
        "url": "https://github.com/gdamore/tree-sitter-d",
        "example": (
            'import std.stdio;\n\nvoid main() {\n    writeln("Hello, World!");\n}\n'
        ),
    },
    "dart": {
        "comment": "//",
        "suffix": "dart",
        "url": "https://github.com/UserNobody14/tree-sitter-dart",
        "example": "void main() {\n  print('Hello, World!');\n}\n",
    },
    "dockerfile": {
        "comment": "#",
        "suffix": "Dockerfile",
        "url": "https://github.com/camdencheek/tree-sitter-dockerfile",
        "example": "FROM ubuntu\n\nRUN echo 'Hello, World!'\n",
    },
    "dot": {
        "comment": "//",
        "suffix": "dot",
        "url": "https://github.com/rydesun/tree-sitter-dot",
        "example": 'graph HelloWorld {\n  label="Hello, World!"\n}\n',
    },
    "elisp": {
        "comment": ";;",
        "suffix": "el",
        "url": "https://github.com/Wilfred/tree-sitter-elisp",
        "example": '(message "Hello, World!")\n',
    },
    "elixir": {
        "comment": "#",
        "suffix": "ex",
        "url": "https://github.com/elixir-lang/tree-sitter-elixir",
        "example": 'IO.puts "Hello, World!"\n',
    },
    "elm": {
        "comment": "--",
        "suffix": "elm",
        "url": "https://github.com/elm-tooling/tree-sitter-elm",
        "example": (
            'import Browser\n\nmain = Browser.sandbox { init = "Hello, World!" }\n'
        ),
    },
    "eno": {
        "comment": "#",
        "suffix": "eno",
        "url": "https://github.com/eno-lang/tree-sitter-eno",
        "example": "message: Hello, World!\n",
    },
    "erlang": {
        "comment": "%",
        "suffix": "erl",
        "url": "https://github.com/WhatsApp/tree-sitter-erlang",
        "example": (
            "-module(hello).\n-export([world/0]).\n\nworld() ->\n"
            '    io:format("Hello, World!~n").\n'
        ),
    },
    "fennel": {
        "comment": ";",
        "suffix": "fnl",
        "url": "https://github.com/travonted/tree-sitter-fennel",
        "example": '(print "Hello, World!")\n',
    },
    "fish": {
        "comment": "#",
        "suffix": "fish",
        "url": "https://github.com/ram02z/tree-sitter-fish",
        "example": 'echo "Hello, World!"\n',
    },
    "formula": {
        "comment": ";",
        "suffix": "formula",
        "url": "https://github.com/siraben/tree-sitter-formula",
        "example": "A1: Hello, World!\n",
    },
    "fortran": {
        "comment": "!",
        "suffix": "f90",
        "url": "https://github.com/stadelmanma/tree-sitter-fortran",
        "example": (
            "program HelloWorld\n  print *, 'Hello, World!'\nend program HelloWorld\n"
        ),
        "functional_node_types": ["function"],
        "comment_node_type": "comment",
    },
    "gitattributes": {
        "comment": "#",
        "suffix": "gitattributes",
        "url": "https://github.com/ObserverOfTime/tree-sitter-gitattributes",
        "example": "* text=auto\n",
    },
    "gitignore": {
        "comment": "#",
        "suffix": "gitignore",
        "url": "https://github.com/shunsambongi/tree-sitter-gitignore",
        "example": "*.log\n",
    },
    "gleam": {
        "comment": "//",
        "suffix": "gleam",
        "url": "https://github.com/gleam-lang/tree-sitter-gleam",
        "example": 'pub fn main() {\n    io.print("Hello, World!")\n}\n',
    },
    "glsl": {
        "comment": "//",
        "suffix": "glsl",
        "url": "https://github.com/theHamsta/tree-sitter-glsl",
        "example": "void main() {\n    gl_FragColor = vec4(1.0, 0.0, 0.0, 1.0);\n}\n",
    },
    "go": {
        "comment": "//",
        "suffix": "go",
        "url": "https://github.com/tree-sitter/tree-sitter-go",
        "example": (
            'package main\n\nimport "fmt"\n\nfunc main() {\n'
            '    fmt.Println("Hello, World!")\n}\n'
        ),
    },
    "graphql": {
        "comment": "#",
        "suffix": "graphql",
        "url": "https://github.com/bkegley/tree-sitter-graphql",
        "example": "query {\n  hello\n}\n",
    },
    "hack": {
        "comment": "//",
        "suffix": "hack",
        "url": "https://github.com/slackhq/tree-sitter-hack",
        "example": '<?hh\n\necho "Hello, World!";\n',
    },
    "haskell": {
        "comment": "--",
        "suffix": "hs",
        "url": "https://github.com/tree-sitter/tree-sitter-haskell",
        "example": 'main :: IO ()\nmain = putStrLn "Hello, World!"\n',
    },
    "hcl": {
        "comment": "#",
        "suffix": "hcl",
        "url": "https://github.com/MichaHoffmann/tree-sitter-hcl",
        "example": 'variable "message" {\n  default = "Hello, World!"\n}\n',
    },
    "html": {
        "comment": "<!--",
        "suffix": "html",
        "url": "https://github.com/tree-sitter/tree-sitter-html",
        "example": (
            "<html>\n<head>\n  <title>Hello, World!</title>\n</head>\n<body>\n  <h1>Hello"
            ", World!</h1>\n</body>\n</html>\n"
        ),
    },
    "ibmhlasm": {
        "comment": "*",
        "suffix": "asm",
        "url": "https://github.com/janus-llm/tree-sitter-ibmhlasm.git",
        "branch": "metrics",
        "example": (
            """
                     TITLE 'Hello, World! Program'
            HELLO    CSECT
                     STM   14,12,12(13)
                     LR    12,15
                     USING *,12
                     LA    15,SAVEAREA
                     ST    13,4(,15)
                     ST    15,8(13)
                     LR    13,15
                     WTO   'Hello, World!'
                     L     13,4(,13)
                     LM    14,12,12(13)
                     LA    15,0
                     BR    14
            SAVEAREA DS    18F
                     END   HELLO
        """
        ),
        "functional_node_types": ["csect", "dsect"],
        "branch_node_types": ["branch_instruction"],
        "operation_node_types": ["operation", "branch_operation"],
        "operand_node_types": ["operands"],
    },
    "java": {
        "comment": "//",
        "suffix": "java",
        "url": "https://github.com/tree-sitter/tree-sitter-java",
        "example": (
            "public class HelloWorld {\n    public static void main(String[] args) {\n"
            '        System.out.println("Hello, World!");\n    }\n}\n'
        ),
    },
    "javascript": {
        "comment": "//",
        "suffix": "js",
        "url": "https://github.com/tree-sitter/tree-sitter-javascript",
        "example": "console.log('Hello, World!');\n",
    },
    "jq": {
        "comment": "#",
        "suffix": "jq",
        "url": "https://github.com/flurie/tree-sitter-jq",
        "example": ".\n",
    },
    "json": {
        "comment": "//",
        "suffix": "json",
        "url": "https://github.com/tree-sitter/tree-sitter-json",
        "example": '{\n  "message": "Hello, World!"\n}\n',
    },
    "json5": {
        "comment": "//",
        "suffix": "json5",
        "url": "https://github.com/Joakker/tree-sitter-json5",
        "example": "{\n  message: 'Hello, World!'\n}\n",
    },
    "julia": {
        "comment": "#",
        "suffix": "jl",
        "url": "https://github.com/tree-sitter/tree-sitter-julia",
        "example": 'println("Hello, World!")\n',
    },
    "kotlin": {
        "comment": "//",
        "suffix": "kt",
        "url": "https://github.com/fwcd/tree-sitter-kotlin",
        "example": 'fun main() {\n    println("Hello, World!")\n}\n',
    },
    "lalrpop": {
        "comment": "//",
        "suffix": "lalrpop",
        "url": "https://github.com/traxys/tree-sitter-lalrpop",
        "example": (
            "grammar Hello {\n    extern {\n        fn print(s: &str);\n    }\n\n"
            "    pub token Integer = r'\\d+';\n    pub rule hello() = \"Hello, \" "
            "Integer;\n}\n"
        ),
    },
    "latex": {
        "comment": "%",
        "suffix": "tex",
        "url": "https://github.com/latex-lsp/tree-sitter-latex",
        "example": (
            "\\documentclass{article}\n\\begin{document}\n  Hello, "
            "World!\n\\end{document}\n"
        ),
    },
    "lean": {
        "comment": "--",
        "suffix": "lean",
        "url": "https://github.com/Julian/tree-sitter-lean",
        "example": 'def main : io io.unit := io.put_str_ln "Hello, World!"\n',
    },
    "llvm": {
        "comment": ";",
        "suffix": "ll",
        "url": "https://github.com/benwilliamgraham/tree-sitter-llvm",
        "example": (
            "; ModuleID = 'hello.ll'\n\ndefine void @main() {\nentry:\n  call void "
            "@puts(i8* getelementptr inbounds ([14 x i8], [14 x i8]* @.str, i32 0, "
            "i32 0))\n  ret void\n}\n\n@.str = private unnamed_addr constant [14 x i8] "
            'c"Hello, World!\\0A\\00", align 1\n'
        ),
    },
    "lua": {
        "comment": "--",
        "suffix": "lua",
        "url": "https://github.com/Azganoth/tree-sitter-lua",
        "example": 'print("Hello, World!")\n',
    },
    "m68k": {
        "comment": ";",
        "suffix": "m68k",
        "url": "https://github.com/grahambates/tree-sitter-m68k",
        "example": "moveq #0,d0\n",
    },
    "markdown": {
        "comment": "<!--",
        "suffix": "md",
        "url": "https://github.com/ikatyang/tree-sitter-markdown",
        "example": "# Hello, World!\n\nThis is a Markdown document.\n",
    },
    "matlab": {
        "comment": "%",
        "suffix": "m",
        "url": "https://github.com/acristoffers/tree-sitter-matlab",
        "example": "fprintf('Hello, World!\\n');\n",
    },
    "meson": {
        "comment": "#",
        "suffix": "meson",
        "url": "https://github.com/staysail/tree-sitter-meson",
        "example": "project('hello', 'c')\n\nexecutable('hello', 'hello.c')\n",
    },
    "mumps": {
        "comment": ";",
        "suffix": "m",
        "url": "https://github.com/janus-llm/tree-sitter-mumps",
        "example": 'WRITE "Hello, World!"',
        "functional_node_types": ["routine_definition"],
        "comment_node_type": "comment",
        "branch_node_types": ["if_statement"],
        "operation_node_types": [
            "command",
            "function_call",
            "routine_call",
            "routine_definition",
        ],
        "operand_node_types": ["argument"],
    },
    "nix": {
        "comment": "#",
        "suffix": "nix",
        "url": "https://github.com/cstrahan/tree-sitter-nix",
        "example": '{ message = "Hello, World!"; }\n',
    },
    "objc": {
        "comment": "//",
        "suffix": "m",
        "url": "https://github.com/jiyee/tree-sitter-objc",
        "example": (
            "#import <Foundation/Foundation.h>\n\nint main() {\n    "
            '@autoreleasepool {\n        NSLog(@"Hello, World!");\n    }\n'
            "    return 0;\n}\n"
        ),
    },
    "ocaml": {
        "comment": "(*",
        "suffix": "ml",
        "url": "https://github.com/tree-sitter/tree-sitter-ocaml",
        "example": 'print_endline "Hello, World!";;\n',
    },
    "org": {
        "comment": "#",
        "suffix": "org",
        "url": "https://github.com/milisims/tree-sitter-org",
        "example": "#+TITLE: Hello, World!\n\nThis is an Org mode document.\n",
    },
    "pascal": {
        "comment": "//",
        "suffix": "pas",
        "url": "https://github.com/Isopod/tree-sitter-pascal",
        "example": "// Hello, World!\nWRITE 'Hello, World!'\n",
    },
    "perl": {
        "comment": "#",
        "suffix": "pl",
        "url": "https://github.com/tree-sitter-perl/tree-sitter-perl",
        "example": "# Hello, World!\nprint 'Hello, World!\\n';\n",
    },
    "pgn": {
        "comment": "%",
        "suffix": "pgn",
        "url": "https://github.com/rolandwalker/tree-sitter-pgn",
        "example": '% Hello, World!\nWRITE "Hello, World!"\n',
    },
    "php": {
        "comment": "#",
        "suffix": "php",
        "url": "https://github.com/tree-sitter/tree-sitter-php",
        "example": "<?php\n// Hello, World!\necho 'Hello, World!';\n",
    },
    "pod": {
        "comment": "=",
        "suffix": "pod",
        "url": "https://github.com/tree-sitter-perl/tree-sitter-pod",
        "example": "=head1 Hello, World!\n\nHello, World!\n\n=cut\n",
    },
    "powershell": {
        "comment": "#",
        "suffix": "ps1",
        "url": "https://github.com/PowerShell/tree-sitter-PowerShell",
        "example": "# Hello, World!\nWrite-Host 'Hello, World!'\n",
    },
    "proto": {
        "comment": "//",
        "suffix": "proto",
        "url": "https://github.com/mitchellh/tree-sitter-proto",
        "example": "// Hello, World!\nmessage HelloWorld {\n  string message = 1;\n}\n",
    },
    "pseudocode": {
        "comment": "",
        "suffix": "txt",
        "url": "",
        "example": "print Hello, World!",
    },
    "python": {
        "comment": "#",
        "suffix": "py",
        "url": "https://github.com/tree-sitter/tree-sitter-python",
        "example": "# Hello, World!\nprint('Hello, World!')\n",
        "functional_node_types": ["function_definition"],
        "comment_node_type": "comment",
    },
    "qmljs": {
        "comment": "//",
        "suffix": "qml",
        "url": "https://github.com/yuja/tree-sitter-qmljs",
        "example": (
            "// Hello, World!\nimport QtQuick 2.0\n\nItem {\n  Text { text: "
            "'Hello, World!' }\n}\n"
        ),
    },
    "r": {
        "comment": "#",
        "suffix": "r",
        "url": "https://github.com/r-lib/tree-sitter-r",
        "example": "# Hello, World!\nprint('Hello, World!')\n",
    },
    "racket": {
        "comment": ";;",
        "suffix": "rkt",
        "url": "https://github.com/6cdh/tree-sitter-racket",
        "example": ';; Hello, World!\n(displayln "Hello, World!")\n',
    },
    "rasi": {
        "comment": "#",
        "suffix": "rasi",
        "url": "https://github.com/Fymyte/tree-sitter-rasi",
        "example": "# Hello, World!\nprint('Hello, World!')\n",
    },
    "re2c": {
        "comment": "//",
        "suffix": "re2c",
        "url": "https://github.com/alemuller/tree-sitter-re2c",
        "example": '// Hello, World!\n{printf("Hello, World!\\n");}',
    },
    "regex": {
        "comment": "#",
        "suffix": "regex",
        "url": "https://github.com/tree-sitter/tree-sitter-regex",
        "example": "# Hello, World!\n# Match 'Hello, World!' with a regex\n",
    },
    "rego": {
        "comment": "#",
        "suffix": "rego",
        "url": "https://github.com/FallenAngel97/tree-sitter-rego",
        "example": '# Hello, World!\npackage hello\n\nmain = {\n  "Hello, World!"\n}\n',
    },
    "rst": {
        "comment": "..",
        "suffix": "rst",
        "url": "https://github.com/stsewd/tree-sitter-rst",
        "example": ".. Hello, World!\n\nHello, World!\n",
    },
    "ruby": {
        "comment": "#",
        "suffix": "rb",
        "url": "https://github.com/tree-sitter/tree-sitter-ruby",
        "example": "# Hello, World!\nputs 'Hello, World!'\n",
    },
    "rust": {
        "comment": "//",
        "suffix": "rs",
        "url": "https://github.com/tree-sitter/tree-sitter-rust",
        "example": '// Hello, World!\nfn main() {\n    println!("Hello, World!");\n}\n',
    },
    "scala": {
        "comment": "//",
        "suffix": "scala",
        "url": "https://github.com/tree-sitter/tree-sitter-scala",
        "example": (
            "// Hello, World!\nobject HelloWorld {\n  def main(args: "
            'Array[String]): Unit = {\n    println("Hello, World!")\n  }\n}\n'
        ),
    },
    "scheme": {
        "comment": ";;",
        "suffix": "scm",
        "url": "https://github.com/6cdh/tree-sitter-scheme",
        "example": ';; Hello, World!\n(displayln "Hello, World!")\n',
    },
    "scss": {
        "comment": "//",
        "suffix": "scss",
        "url": "https://github.com/serenadeai/tree-sitter-scss",
        "example": "// Hello, World!\n$variable: 'Hello, World!';\n",
    },
    "sexp": {
        "comment": ";",
        "suffix": "sexp",
        "url": "https://github.com/AbstractMachinesLab/tree-sitter-sexp",
        "example": '; Hello, World!\n(println "Hello, World!")\n',
    },
    "sfapex": {
        "comment": "//",
        "suffix": "cls",
        "url": "https://github.com/aheber/tree-sitter-sfapex",
        "example": (
            "// Hello, World!\npublic class HelloWorld {\n    public static void "
            'main(String[] args) {\n        System.out.println("Hello, World!");\n    '
            "}\n}\n"
        ),
    },
    "smali": {
        "comment": "#",
        "suffix": "smali",
        "url": "https://github.com/amaanq/tree-sitter-smali",
        "example": (
            ".class public LHelloWorld;\n.super Ljava/lang/Object;\n\n.method "
            "public static main([Ljava/lang/String;)V\n    .registers 2\n\n    "
            "sget-object v0, Ljava/lang/System;->out:Ljava/io/PrintStream;\n    "
            'const-string v1, "Hello, World!"\n    invoke-virtual {v0, v1}, '
            "Ljava/io/PrintStream;->println(Ljava/lang/String;)V\n\n    "
            "return-void\n.end method\n"
        ),
    },
    "sourcepawn": {
        "comment": "//",
        "suffix": "sp",
        "url": "https://github.com/nilshelmig/tree-sitter-sourcepawn",
        "example": (
            "#include <sourcemod>\n\npublic Plugin:myinfo = \n{\n    name    = "
            '"Hello World",\n    author  = "You",\n    version = "1.0",\n    url     = '
            '"",\n    description = "",\n};\n\nvoid OnPluginStart()\n{\n    '
            'PrintToServer("Hello, World!");\n}\n'
        ),
    },
    "sparql": {
        "comment": "#",
        "suffix": "sparql",
        "url": "https://github.com/BonaBeavis/tree-sitter-sparql",
        "example": 'SELECT "Hello, World!"',
    },
    "sql": {
        "comment": "--",
        "suffix": "sql",
        "url": "https://github.com/m-novikov/tree-sitter-sql",
        "example": "SELECT 'Hello, World!';",
    },
    "sqlite": {
        "comment": "--",
        "suffix": "sqlite",
        "url": "https://github.com/dhcmrlchtdj/tree-sitter-sqlite",
        "example": "SELECT 'Hello, World!';",
    },
    "svelte": {
        "comment": "<!--",
        "suffix": "svelte",
        "url": "https://github.com/Himujjal/tree-sitter-svelte",
        "example": "<script>\n  console.log('Hello, World!');\n</script>\n",
    },
    "swift": {
        "comment": "//",
        "suffix": "swift",
        "url": "https://github.com/alex-pinkus/tree-sitter-swift",
        "example": 'print("Hello, World!")\n',
    },
    "systemrdl": {
        "comment": "//",
        "suffix": "systemrdl",
        "url": "https://github.com/SystemRDL/tree-sitter-systemrdl",
        "example": (
            "address_space {\n  reg my_register {\n    address 0x0;\n    "
            "field my_field {\n      bits 0;\n      access r; // Read-only\n"
            "      reset 0;\n      hardware_bug; // It's a feature!\n    }\n  }\n}\n"
        ),
    },
    "template": {
        "comment": "<%#",
        "suffix": "txt",
        "url": "https://github.com/tree-sitter/tree-sitter-embedded-template",
        "example": "<%# Hello, World! %>\nHello, World!\n",
    },
    "text": {
        "comment": "",
        "suffix": "txt",
        "url": None,
        "example": "Hello, World!",
    },
    "thrift": {
        "comment": "//",
        "suffix": "thrift",
        "url": "https://github.com/duskmoon314/tree-sitter-thrift",
        "example": (
            "namespace py tutorial\n\nconst string HELLO_URL = "
            '"http://example.com/hello"\n\nstruct HelloRequest {}\n\nstruct '
            "HelloResponse {}\n\nservice HelloWorld {\n  HelloResponse hello(1: "
            "HelloRequest request)\n}\n"
        ),
    },
    "toml": {
        "comment": "#",
        "suffix": "toml",
        "url": "https://github.com/ikatyang/tree-sitter-toml",
        "example": 'message = "Hello, World!"\n',
    },
    "turtle": {
        "comment": "#",
        "suffix": "ttl",
        "url": "https://github.com/BonaBeavis/tree-sitter-turtle",
        "example": (
            "@prefix ex: <http://example.org/> .\n\nex:hello ex:message "
            '"Hello, World!" .\n'
        ),
    },
    "twig": {
        "comment": "{#",
        "suffix": "twig",
        "url": "https://github.com/gbprod/tree-sitter-twig",
        "example": "{# Hello, World! #}\nHello, World!\n",
    },
    "typescript": {
        "comment": "//",
        "suffix": "ts",
        "url": "https://github.com/tree-sitter/tree-sitter-typescript",
        "example": "console.log('Hello, World!');\n",
    },
    "uml": {
        "comment": "'",
        "suffix": "uml",
        "url": "https://github.com/lyndsysimon/tree-sitter-plantuml",
        "example": "@startuml\nAlice -> Bob: Authentication Request\nBob --> Alice:\
              Authentication Response\nAlice -> Bob: Another authentication\
              Request\nAlice <-- Bob: Another authentication Response\n@enduml",
    },
    "verilog": {
        "comment": "//",
        "suffix": "v",
        "url": "https://github.com/tree-sitter/tree-sitter-verilog",
        "example": (
            "module HelloWorld;\n  initial begin\n    "
            '$display("Hello, World!");\n  end\nendmodule\n'
        ),
    },
    "vhdl": {
        "comment": "--",
        "suffix": "vhdl",
        "url": "https://github.com/alemuller/tree-sitter-vhdl",
        "example": (
            "library IEEE;\nuse IEEE.STD_LOGIC_1164.ALL;\n\nentity "
            "HelloWorld is\nend HelloWorld;\n\narchitecture Behavioral of HelloWorld is"
            '\nbegin\n  process\n  begin\n    report "Hello, World!";\n    wait;\n  end '
            "process;\nend Behavioral;\n"
        ),
    },
    "vue": {
        "comment": "//",
        "suffix": "vue",
        "url": "https://github.com/ikatyang/tree-sitter-vue",
        "example": "<template>\n  <div>\n    Hello, World!\n  </div>\n</template>\n",
    },
    "wasm": {
        "comment": ";;",
        "suffix": "wasm",
        "url": "https://github.com/wasm-lsp/tree-sitter-wasm",
        "example": (
            '(module\n  (func $main (export "main")\n    (import "env" "puts" '
            "(func $puts (param i32)))\n    (call $puts (i32.const 0))\n  )\n)"
        ),
    },
    "wgsl": {
        "comment": "//",
        "suffix": "wgsl",
        "url": "https://github.com/mehmetoguzderin/tree-sitter-wgsl",
        "example": (
            "[[stage(fragment)]]\nfn main() -> [[builtin(position)]] "
            "vec4<f32> {\n    return vec4<f32>(1.0, 0.0, 0.0, 1.0);\n}\n"
        ),
    },
    "yaml": {
        "comment": "#",
        "suffix": "yaml",
        "url": "https://github.com/ikatyang/tree-sitter-yaml",
        "example": "message: Hello, World!\n",
    },
    "yang": {
        "comment": "//",
        "suffix": "yang",
        "url": "https://github.com/Hubro/tree-sitter-yang",
        "example": (
            "module hello-world {\n  yang-version 1.1;\n  namespace "
            '"urn:example:hello-world";\n\n  prefix hw;\n\n  container hello-world {\n'
            '    leaf message {\n      type string;\n      default "Hello, World!";\n'
            "    }\n  }\n}\n"
        ),
    },
    "zig": {
        "comment": ";",
        "suffix": "zig",
        "url": "https://github.com/maxxnino/tree-sitter-zig",
        "example": (
            'const std = @import("std");\n\npub fn main() void {\n    const '
            'stdout = std.io.getStdOut();\n    stdout.print("Hello, World!\\n");\n}\n'
        ),
    },
}



from janus.language.block import CodeBlock
from janus.language.naive.registry import register_splitter
from janus.language.node import NodeType
from janus.language.splitter import Splitter


@register_splitter("chunk")
class ChunkSplitter(Splitter):
    """
    Splits into fixed chunk sizes without parsing
    """

    def _get_ast(self, code: str) -> CodeBlock:
        return CodeBlock(
            text=code,
            name="root",
            id="root",
            start_point=(0, 0),
            end_point=(code.count("\n"), 0),
            start_byte=0,
            end_byte=len(bytes(code, "utf-8")),
            node_type=NodeType("program"),
            children=[],
            language=self.language,
            tokens=self._count_tokens(code),
        )



import os
import unittest
from pathlib import Path
from unittest.mock import patch

import pytest

from janus.language.binary import BinarySplitter
from janus.llm import load_model


class TestBinarySplitter(unittest.TestCase):
    """Tests for the BinarySplitter class."""

    def setUp(self):
        model_name = "gpt-4o"
        self.binary_file = Path("janus/language/binary/_tests/hello")
        self.llm, _, _, _ = load_model(model_name)
        self.splitter = BinarySplitter(model=self.llm)
        os.environ["GHIDRA_INSTALL_PATH"] = "~/programs/ghidra_10.4_PUBLIC"

    def test_setup(self):
        """Test that the setup sets the environment variable correctly."""
        with patch("os.getenv") as mock_getenv:
            mock_getenv.return_value = "~/programs/ghidra_10.4_PUBLIC"
            self.assertEqual(
                os.getenv("GHIDRA_INSTALL_PATH"), "~/programs/ghidra_10.4_PUBLIC"
            )
            mock_getenv.assert_called_once_with("GHIDRA_INSTALL_PATH")

    def test_initialization(self):
        """Test that BinarySplitter is initialized correctly."""
        self.assertIsInstance(self.splitter, BinarySplitter)
        self.assertEqual(self.splitter.model, self.llm)

    @pytest.mark.ghidra(
        reason=(
            "No way to test this in CI w/o installing Ghidra, but want to keep here to "
            "run manually."
        )
    )
    def test__get_decompilation(self) -> None:
        """Test that _get_decompilation returns a string."""
        decompiled: str = (
            '\nundefined4 entry(void)\n\n{\n  _printf("Hello, World!");\n  '
            "return 0;\n}\n\n"
        )
        self.assertEqual(self.splitter._get_decompilation(self.binary_file), decompiled)



from janus.utils.enums import LANGUAGES
from janus.utils.logger import create_logger

log = create_logger(__name__)


class FileManager:
    """A parent class that manages the files that are being translated."""

    def __init__(self, language: str = "python") -> None:
        """Initialize a FileManager instance.

        Arguments:
            language: The name of the language to manage.
        """
        self.language: str = language
        self.comment: str = LANGUAGES[self.language]["comment"]
        self.suffix: str = LANGUAGES[self.language]["suffix"]



import glob
import os.path

modules = glob.glob(os.path.join(os.path.dirname(__file__), "*.py"))
__all__ = []
for m in modules:
    if os.path.isfile(m) and not os.path.samefile(m, __file__):
        __all__.append(os.path.basename(m)[:-3])



import json
import re

from langchain.output_parsers.json import parse_json_markdown
from langchain.schema.output_parser import BaseOutputParser
from langchain_core.exceptions import OutputParserException
from langchain_core.messages import AIMessage

from janus.language.block import CodeBlock
from janus.parsers.code_parser import JanusParser
from janus.utils.logger import create_logger

log = create_logger(__name__)


class RequirementsParser(BaseOutputParser[str], JanusParser):
    block_name: str = ""

    def __init__(self):
        super().__init__(expected_keys=[])

    def set_reference(self, block: CodeBlock):
        self.block_name = block.name

    def parse(self, text: str) -> str:
        if isinstance(text, AIMessage):
            text = text.content
        text = text.lstrip("```json")
        text = text.rstrip("```")
        try:
            obj = parse_json_markdown(text)
        except json.JSONDecodeError as e:
            log.debug(f"Invalid JSON object. Output:\n{text}")
            raise OutputParserException(f"Got invalid JSON object. Error: {e}")

        if not isinstance(obj, dict):
            raise OutputParserException(
                f"Got invalid return object. Expected a dictionary, but got {type(obj)}"
            )
        return json.dumps(obj)

    def parse_combined_output(self, text: str):
        """Parse the output text from the LLM when multiple inputs are combined.

        Arguments:
            text: The output text from the LLM.

        Returns:
            A parsed version of the text.
        """
        json_strings = re.findall(r"\{.*?\}", text)
        output_list = list()
        for i, json_string in enumerate(json_strings, 1):
            json_dict = json.loads(json_string)
            output_list.append(json_dict["requirements"])
        return output_list

    def get_format_instructions(self) -> str:
        """Get the format instructions for the parser.

        Returns:
            The format instructions for the LLM.
        """
        return (
            "Output must contain a requirements specification "
            "in a JSON-formatted string. The only key should be "
            "'requirements' and its value should be a JSON-formatted list "
            "containing the requirements."
        )

    @property
    def _type(self) -> str:
        return self.__class__.name



from typing import Iterable

from rich.progress import (
    BarColumn,
    MofNCompleteColumn,
    Progress,
    SpinnerColumn,
    TextColumn,
    TimeElapsedColumn,
    TimeRemainingColumn,
)

from janus.utils.logger import create_logger

log = create_logger(__name__)


def track(
    iterable: Iterable, description: None | str = None, total: None | int = None
) -> Iterable:
    """Track progress of an iterable.

    Arguments:
        iterable: The iterable to track.
        description: The description to show for the progress bar.
        total: The total number of items in the iterable.
    """
    if total is None:
        try:
            total = len(iterable)
        except TypeError:
            log.debug("Could not determine total number of items in iterable.")
            total = None

    with Progress(
        SpinnerColumn(),
        TextColumn("[bold blue]{task.description}", justify="right"),
        BarColumn(),
        MofNCompleteColumn(),
        "[progress.percentage]{task.percentage:>3.0f}%",
        "[yellow]Time Elapsed:",
        TimeElapsedColumn(),
        "[cyan]Time Remaining:",
        TimeRemainingColumn(),
    ) as progress:
        task = progress.add_task(description, total=total)
        for item in iterable:
            progress.update(task, advance=1)
            yield item


if __name__ == "__main__":
    from time import sleep

    for i in track(range(100), description="Processing:"):
        sleep(0.5)



import traceback
import unittest
from unittest.mock import ANY, patch

from typer.testing import CliRunner

from janus.cli import app, translate
from janus.embedding.embedding_models_info import EMBEDDING_MODEL_CONFIG_DIR
from janus.llm.models_info import MODEL_CONFIG_DIR


class TestCli(unittest.TestCase):
    def setUp(self):
        self.runner = CliRunner()

    def test_help(self):
        result = self.runner.invoke(app, ["--help"])
        self.assertEqual(result.exit_code, 0)

    def test_version(self):
        result = self.runner.invoke(app, ["--version"])
        self.assertEqual(result.exit_code, 0)

    def test_invalid_command(self):
        result = self.runner.invoke(app, ["invalid_command"])
        self.assertNotEqual(result.exit_code, 0)

    def test_no_arguments(self):
        result = self.runner.invoke(app)
        self.assertEqual(result.exit_code, 0)

    def test_translate_help(self):
        result = self.runner.invoke(app, ["translate", "-h"])
        self.assertEqual(result.exit_code, 0)

    def test_document_help(self):
        result = self.runner.invoke(app, ["document", "-h"])
        self.assertEqual(result.exit_code, 0)

    def test_db_help(self):
        result = self.runner.invoke(app, ["db", "-h"])
        self.assertEqual(result.exit_code, 0)

    def test_llm_help(self):
        result = self.runner.invoke(app, ["llm", "-h"])
        self.assertEqual(result.exit_code, 0)

    def test_llm_add(self):
        llm_model_path = MODEL_CONFIG_DIR / "test-model-name.json"
        if llm_model_path.exists():
            llm_model_path.unlink()
        result = self.runner.invoke(app, ["llm", "add", "test-model-name"])
        self.assertEqual(result.exit_code, 0)
        self.assertTrue(llm_model_path.exists())
        llm_model_path.unlink()

    def test_db_ls(self):
        result = self.runner.invoke(app, ["db", "ls"])
        self.assertEqual(result.exit_code, 0)

    def test_db_status(self):
        result = self.runner.invoke(app, ["db", "status"])
        self.assertEqual(result.exit_code, 0)

    def test_embedding_add(self):
        embedding_model_path = (
            EMBEDDING_MODEL_CONFIG_DIR / "test-embedding-model-name.json"
        )
        if embedding_model_path.exists():
            embedding_model_path.unlink()
        result = self.runner.invoke(
            app, ["embedding", "add", "test-embedding-model-name"]
        )
        self.assertEqual(result.exit_code, 0)
        self.assertTrue(embedding_model_path.exists())
        embedding_model_path.unlink()

    def test_db_add_and_rm(self):
        embedding_model_path = (
            EMBEDDING_MODEL_CONFIG_DIR / "test-embedding-model-name.json"
        )
        if embedding_model_path.exists():
            embedding_model_path.unlink()
        result = self.runner.invoke(
            app,
            ["embedding", "add", "test-embedding-model-name", "-t", "HuggingFaceLocal"],
        )
        self.assertEqual(result.exit_code, 0)
        result = self.runner.invoke(
            app,
            [
                "db",
                "add",
                "test-db-name",
                "test-embedding-model-name",
                "-i",
                "janus/language/mumps",
            ],
        )
        traceback.print_exception(result.exception)
        embedding_model_path.unlink()
        self.assertEqual(result.exit_code, 0)

        result = self.runner.invoke(app, ["db", "rm", "test-db-name", "-y"])
        self.assertEqual(result.exit_code, 0)

    @patch("janus.converter.translate.Translator.translate", autospec=True)
    def test_translate(self, mock_translate):
        # Arrange
        mock_instance = mock_translate.return_value
        mock_instance.translate.return_value = None  # or whatever you expect

        # Act
        translate(
            source_lang="matlab",
            target_lang="python",
            input_dir="janus/",
            output_dir="janus/",
            overwrite=True,
            temp=0.7,
            prompt_template="simple",
            collection=None,
        )

        # Assert
        mock_translate.assert_called_once()
        mock_translate.assert_called_once_with(ANY, "janus/", "janus/", True, None)



import unittest

from sacrebleu import sentence_chrf

from janus.metrics.chrf import chrf


class TestChrF(unittest.TestCase):
    def setUp(self):
        self.target_text = "This is a source text."
        self.reference_text = "This is a destination text."
        self.char_order = 6
        self.word_order = 2
        self.beta = 2.0

    def test_chrf_custom_params(self):
        """Test the chrf function with custom parameters."""
        function_score = chrf(
            self.target_text,
            self.reference_text,
            self.char_order,
            self.word_order,
            self.beta,
        )
        score = sentence_chrf(
            hypothesis=self.target_text,
            references=[self.reference_text],
            char_order=self.char_order,
            word_order=self.word_order,
            beta=self.beta,
        )
        expected_score = float(score.score) / 100.0
        self.assertEqual(function_score, expected_score)

    def test_chrf_with_s_flag(self):
        """Test the CHRF score calculation with the -S flag."""
        function_score = sentence_chrf(
            hypothesis=self.target_text,
            references=[self.reference_text],
            char_order=self.char_order,
            word_order=self.word_order,
            beta=self.beta,
        )
        function_score = float(function_score.score) / 100.0
        score_with_s_flag = chrf(
            self.target_text,
            self.reference_text,
            self.char_order,
            self.word_order,
            self.beta,
            use_strings=True,  # Mimics -S
        )
        self.assertEqual(function_score, score_with_s_flag)

    def test_chrf_invalid_target_type(self):
        """Test the chrf function with invalid source text type."""
        with self.assertRaises(TypeError):
            chrf(123, self.reference_text, self.char_order, self.word_order, self.beta)

    def test_chrf_invalid_reference_type(self):
        """Test the chrf function with invalid destination text type."""
        with self.assertRaises(TypeError):
            chrf(self.target_text, 123, self.char_order, self.word_order, self.beta)


if __name__ == "__main__":
    unittest.main()



import uuid
from abc import ABC, abstractmethod
from pathlib import Path
from typing import Any, Dict, Optional, Sequence

from chromadb import Client, Collection
from langchain_community.vectorstores import Chroma

from janus.embedding.collections import Collections
from janus.embedding.database import ChromaEmbeddingDatabase
from janus.language.block import CodeBlock, TranslatedCodeBlock
from janus.utils.enums import EmbeddingType


class Vectorizer(object):
    """Class for creating embeddings/vectors in a specified ChromaDB"""

    def __init__(self, client: Client, config: Optional[Dict[str, Any]] = None) -> None:
        """Initializes the Vectorizer class

        Arguments:
            client: ChromaDB client instance
        """
        self._db = client
        self._collections = Collections(self._db, config)

    def get_or_create_collection(
        self, name: EmbeddingType | str, model_name: Optional[str] = None
    ) -> Chroma:
        return self._collections.get_or_create(name, model_name=model_name)

    def create_collection(
        self, embedding_type: EmbeddingType, model_name: Optional[str] = None
    ) -> Chroma:
        return self._collections.create(embedding_type, model_name=model_name)

    def collections(
        self, name: None | EmbeddingType | str = None
    ) -> Sequence[Collection]:
        return self._collections.get(name)

    def add_nodes_recursively(
        self, code_block: CodeBlock, collection_name: EmbeddingType | str, file_name: str
    ) -> None:
        """Embed all nodes in the tree rooted at `code_block`

        Arguments:
            code_block: CodeBlock to embed
            collection_name: Collection to add to
            file_name: Name of file containing `code_block`
        """
        nodes = [code_block]
        while nodes:
            node = nodes.pop(0)
            self._add(node, collection_name, file_name)
            nodes.extend(node.children)

    def _add(
        self,
        code_block: CodeBlock,
        collection_name: EmbeddingType | str,
        filename: str,  # perhaps this should be a relative path from the source, but for
        # now we're all in 1 directory
    ) -> None:
        """Calculate `code_block` embedding, returning success & storing in `embedding_id`

        Arguments:
            code_block: CodeBlock to embed
            collection_name: Collection to add to
            filename: Name of file containing `code_block`
        """
        if code_block.text:
            metadatas = [
                {
                    "type": code_block.node_type,
                    "id": code_block.id,
                    "name": code_block.name,
                    "language": code_block.language,
                    "filename": filename,
                    "tokens": code_block.tokens,
                    "cost": 0,  # TranslatedCodeBlock has cost
                },
            ]
            if collection_name in self.config:
                metadatas[0]["embedding_model"] = self.config[collection_name]
            # for now, dealing with missing metadata by skipping it
            if isinstance(code_block, TranslatedCodeBlock):
                self._add(
                    code_block=code_block.original,
                    collection_name=collection_name,
                    filename=filename,
                )
                if code_block.original.embedding_id is not None:
                    metadatas[0][
                        "original_embedding_id"
                    ] = code_block.original.embedding_id
                metadatas[0]["cost"] = code_block.cost
            if code_block.text is not None:
                metadatas[0]["hash"] = hash(code_block.text)
            if code_block.start_point is not None:
                metadatas[0]["start_line"] = code_block.start_point[0]
            if code_block.end_point is not None:
                metadatas[0]["end_line"] = code_block.end_point[0]
            # TODO: Add metadata about translation parameters (e.g. model)
            the_text = [code_block.text]
            code_block.embedding_id = self.add_text(
                collection_name,
                the_text,
                metadatas,
            )[0]

    def add_text(
        self,
        collection_name: EmbeddingType | str,
        texts: list[str],
        metadatas: list[dict],
        ids: list[str] = None,
    ) -> list[str]:
        """Helper function that stores a single text (in an array) and associated
        metadatas, returning the embedding id

        Arguments:
            collection_name: Collection to add to
            texts: list of texts to store
            metadatas: list of metadatas to store
            ids: list of embedding ids (must match lengh of texts),
                 generated if not given by caller

        Returns:
            list of embedding ids. Raises ValueError if collection not found.
        """
        if ids is None:
            # Logic originally from langchain's vectorstores.chroma.Chroma.add_texts
            # Modified to use uuid3 instead of uuid1. We don't need to update
            # an entry in the database if the text is the same. So we generate the UUID
            # based on the text.
            ids = [str(uuid.uuid3(uuid.NAMESPACE_DNS, text)) for text in texts]
        collection = self._collections.get_or_create(collection_name)
        collection.add_texts(ids=ids, texts=texts, metadatas=metadatas)
        return ids

    @property
    def config(self):
        return self._collections._config


class VectorizerFactory(ABC):
    """Interface for creating a Vectorizer independent of type of ChromaDB client"""

    @abstractmethod
    def create_vectorizer(
        self, path: str | Path, config: Dict[str, Any] = {}
    ) -> Vectorizer:
        """Factory method"""


class ChromaDBVectorizer(VectorizerFactory):
    """Factory for Vectorizer that uses ChromaEmbeddingDatabase"""

    def create_vectorizer(
        self,
        path: str | Path = Path.home() / ".janus" / "chroma" / "chroma-data",
        config: Optional[Dict[str, Any]] = None,
    ) -> Vectorizer:
        """
        Arguments:
            path: The path to the ChromaDB. Can be either a string of a URL or path or a
                Path object

            Returns:
                Vectorizer
        """
        database = ChromaEmbeddingDatabase(path)
        return Vectorizer(database, config)



from typing import Callable

SPLITTING_METHODS: dict[str, Callable[[str, str], list[str]]] = {}


def register_splitting_method(name: None | str = None) -> Callable[[Callable], Callable]:
    """Registers a pairing method for splitting strings in files

    Arguments:
        name: The name of the splitting method. If None, the function name is used.
        help: The help text for the pairing method.

    Returns:
        The decorator function.
    """

    def decorator(f: Callable[[str, str], list[tuple[str, str]]]):
        if name is None:
            splitting_name = f.__name__
        else:
            splitting_name = name
        SPLITTING_METHODS[splitting_name] = f
        return f

    return decorator


@register_splitting_method(name="file")
def split_by_file(src: str, **kwargs) -> list[str]:
    """Split the source text by file

    Arguments:
        src: The source text.

    Returns:
        A list of strings.
    """
    return [src]



import datetime
import unittest
from unittest.mock import MagicMock

import pytest

from janus.embedding.collections import Collections
from janus.utils.enums import EmbeddingType


class TestCollections(unittest.TestCase):
    def setUp(self):
        self._db = MagicMock()
        self.collections = Collections(self._db)

    def test_creation(self):
        self.collections.create(EmbeddingType.PSEUDO)

        metadata = {
            "date_updated": datetime.datetime.now().date().isoformat(),
            "time_updated": datetime.datetime.now().time().isoformat("minutes"),
        }

        self._db.create_collection.assert_called_with("pseudo_1", metadata=metadata)

    def test_creation_triangulation(self):
        self.collections.create(EmbeddingType.REQUIREMENT)

        metadata = {
            "date_updated": datetime.datetime.now().date().isoformat(),
            "time_updated": datetime.datetime.now().time().isoformat("minutes"),
        }

        self._db.create_collection.assert_called_with("requirement_1", metadata=metadata)

    def test_creation_of_existing_type(self):
        mock_collection = MagicMock()
        mock_collection.name.__eq__._mock_return_value = True
        self._db.list_collections.return_value = [mock_collection]

        self.collections.create(EmbeddingType.REQUIREMENT)
        metadata = {
            "date_updated": datetime.datetime.now().date().isoformat(),
            "time_updated": datetime.datetime.now().time().isoformat("minutes"),
        }
        self._db.create_collection.assert_called_with("requirement_2", metadata=metadata)

    def test_missing_embedding_type(self):
        with pytest.raises(ValueError):
            self.collections.create(EmbeddingType(1337))

    def test_get_with_None_name(self):
        self._db.list_collections.return_value = 1337

        result = self.collections.get()

        self.assertTrue(self._db.list_collections.called)
        self.assertEqual(result, 1337)
        self.assertFalse(self._db.get_collection.called)

    def test_get_with_embedding_type(self):
        self._db.get_collection.return_value = "blah"

        result = self.collections.get(EmbeddingType.SOURCE)

        self._db.get_collection.assert_called_once_with("source")
        self.assertEqual(result, ["blah"])
        self.assertFalse(self._db.list_collections.called)

    def test_get_with_string_name(self):
        self._db.get_collection.return_value = "xxx"

        result = self.collections.get("foo")

        self._db.get_collection.assert_called_with("foo")
        self.assertEqual(result, ["xxx"])
        self.assertFalse(self._db.list_collections.called)



from janus.language.naive.basic_splitter import FileSplitter
from janus.language.naive.chunk_splitter import ChunkSplitter
from janus.language.naive.simple_ast import get_flexible_ast, get_strict_ast
from janus.language.naive.tag_splitter import TagSplitter



from janus.converter.converter import Converter
from janus.converter.diagram import DiagramGenerator
from janus.converter.document import Documenter, MadLibsDocumenter, MultiDocumenter
from janus.converter.evaluate import Evaluator
from janus.converter.requirements import RequirementsDocumenter
from janus.converter.translate import Translator



import logging
import os

from rich.logging import RichHandler


class LogFilter(logging.Filter):
    """
    Print all messages that do not include "dealloc" (GPU warning message).
    """

    def filter(self, record: logging.LogRecord):
        # This is a `numba` warning that is not useful to print
        # Probably wouldn't come up in this project, but keeping it in here as an example
        if "dealloc" in str(getattr(record, "msg")):
            return False
        elif "That model is currently overloaded" in str(getattr(record, "msg")):
            return False
        elif "Batches: " in str(getattr(record, "msg")):
            return False
        elif "Using default tokenizer." in str(getattr(record, "msg")):
            return False
        elif "Load pretrained SentenceTransformer" in str(getattr(record, "msg")):
            return False
        elif "Use pytorch device" in str(getattr(record, "msg")):
            return False
        elif "creating" in str(getattr(record, "msg")):
            return False
        elif "cc -f" in str(getattr(record, "msg")):
            return False
        elif "c++" in str(getattr(record, "msg")):
            return False
        elif "error_code=context_length_exceeded" in str(getattr(record, "msg")):
            return False
        elif "NumExpr" in str(getattr(record, "msg")):
            return False
        elif "cc -shared" in str(getattr(record, "msg")):
            return False
        return True

    def __repr__(self):
        return "LogFilter"


def create_logger(name: str) -> logging.Logger:
    """Create a `Logger` to be used in different modules

    Parameters:
    -----------
    name: str
        The name of the logger. Will usually be passed in from the module as `__name__`.

    Returns:
    --------
    log: logging.Logger
        The `Logger` object that will be used to create log statements in the terminal.
    """
    if (log_level := os.environ.get("LOGLEVEL")) is None:
        log_level = "INFO"

    FORMAT = "%(message)s"
    rh = RichHandler()
    rh.addFilter(LogFilter())
    handlers = [rh]
    logging.basicConfig(
        level=log_level,
        format=FORMAT,
        datefmt="[%m/%d/%Y %I:%M:%S %p]",
        handlers=handlers,
    )

    log = logging.getLogger(name)
    log.setLevel(log_level)

    return log



import inspect
import json
from pathlib import Path
from typing import Callable, Optional

import click
import typer
from typing_extensions import Annotated

from janus.llm import load_model
from janus.metrics.cli import evaluate
from janus.metrics.file_pairing import FILE_PAIRING_METHODS
from janus.metrics.splitting import SPLITTING_METHODS
from janus.utils.enums import LANGUAGES
from janus.utils.logger import create_logger
from janus.utils.progress import track

log = create_logger(__name__)


def metric(
    name: None | str = None,
    help: None | str = None,
    use_reference: bool = True,
) -> Callable:
    """Returns a decorator to add a given metric to the cli

    Metrics must follow the format (src_str, cmp_str, **other_params)

    Arguments:
        name: The name of the metric. If None, the function name is used.
        help: The help text for the metric.
        use_reference: Whether the metric requires a reference string.

    Returns:
        The decorator function.
    """

    def decorator(function):
        if use_reference:

            def func(
                out_file: Annotated[
                    str,
                    typer.Option("--out-file", "-o", help="Output JSON file to write."),
                ],
                language: Annotated[
                    Optional[str],
                    typer.Option(
                        "--language",
                        "-l",
                        help="The language of the source code.",
                        click_type=click.Choice(sorted(LANGUAGES)),
                    ),
                ] = None,
                target: Annotated[
                    Optional[str],
                    typer.Option(
                        "--target",
                        "-t",
                        help="Target file or string to evaluate.",
                    ),
                ] = None,
                reference: Annotated[
                    Optional[str],
                    typer.Option(
                        "--reference",
                        "-r",
                        help="Reference file or string to use as reference/baseline.",
                    ),
                ] = None,
                json_file_name: Annotated[
                    Optional[str],
                    typer.Option(
                        "--json",
                        "-j",
                        help="Json file to extract pairs from \
                            (if set ignores --target and --reference)",
                    ),
                ] = None,
                target_key: Annotated[
                    str,
                    typer.Option(
                        "--target-key",
                        "-tk",
                        help="json key to extract list of target strings",
                    ),
                ] = "target",
                reference_key: Annotated[
                    str,
                    typer.Option(
                        "--reference-key",
                        "-rk",
                        help="json key to extract list of reference strings",
                    ),
                ] = "reference",
                file_pairing_method: Annotated[
                    str,
                    typer.Option(
                        "--method",
                        "-m",
                        click_type=click.Choice(FILE_PAIRING_METHODS.keys()),
                        help="Method to use for pairing\
                              segments of target and reference files \
                                (ignored for json).",
                    ),
                ] = "file",
                llm_name: Annotated[
                    str,
                    typer.Option(
                        "--llm",
                        "-L",
                        help="The custom name of the model set with 'janus llm add'.",
                    ),
                ] = "gpt-4o",
                progress: Annotated[
                    bool,
                    typer.Option(
                        "--progress",
                        "-p",
                        help="Whether to display a progress bar.",
                        is_flag=True,
                    ),
                ] = False,
                use_strings: Annotated[
                    bool,
                    typer.Option(
                        "--string",
                        "-S",
                        help="Indicate that the target and reference are strings",
                        is_flag=True,
                    ),
                ] = False,
                *args,
                **kwargs,
            ):
                out = []
                llm, _, token_limit, model_cost = load_model(llm_name)
                if json_file_name is not None:
                    with open(json_file_name, "r") as f:
                        json_obj = json.load(f)
                    pairs = {}
                    for key in json_obj:
                        doc = json_obj[key]
                        ref = doc[reference_key]
                        experiments = doc["experiments"]
                        for model_key in experiments:
                            model_dict = experiments[model_key]
                            if not isinstance(model_dict, dict):
                                continue
                            if target_key not in model_dict:
                                continue
                            if model_key not in pairs:
                                pairs[model_key] = {}
                            for k in model_dict[target_key]:
                                pairs[model_key][k] = (model_dict[target_key][k], ref[k])
                elif target is not None and reference is not None:
                    if use_strings:
                        target_contents = target
                        reference_contents = reference
                    else:
                        with open(target, "r") as f:
                            target_contents = f.read()
                        with open(reference, "r") as f:
                            reference_contents = f.read()
                    pairs = FILE_PAIRING_METHODS[file_pairing_method](
                        target_contents,
                        reference_contents,
                        target_file=None if use_strings else target,
                        reference_file=None if use_strings else reference,
                        out_file=out_file,
                        lang=language,
                        llm=llm,
                        token_limit=token_limit,
                        model_cost=model_cost,
                    )
                else:
                    raise ValueError(
                        "Error, specify json or target and reference files/strings"
                    )
                if isinstance(pairs, dict):
                    out = {}
                    for k in pairs:
                        out[k] = apply_function_pairs(
                            pairs[k],
                            function,
                            progress,
                            language,
                            llm,
                            token_limit,
                            model_cost,
                            *args,
                            **kwargs,
                        )
                else:
                    out = apply_function_pairs(
                        pairs,
                        function,
                        progress,
                        language,
                        llm,
                        token_limit,
                        model_cost,
                        *args,
                        **kwargs,
                    )
                out_file = Path(out_file)
                out_file.parent.mkdir(parents=True, exist_ok=True)
                with open(out_file, "w") as f:
                    json.dump(out, f)
                    log.info(f"Saved results to file: {out_file}")

            sig1 = inspect.signature(function)
            sig2 = inspect.signature(func)
            func.__signature__ = sig2.replace(
                parameters=tuple(
                    list(sig2.parameters.values())[:11]
                    + list(sig1.parameters.values())[2:-1]
                )
            )
        else:

            def func(
                out_file: Annotated[
                    str,
                    typer.Option("--out-file", "-o", help="Output JSON file to write."),
                ],
                language: Annotated[
                    Optional[str],
                    typer.Option(
                        "--language",
                        "-l",
                        help="The language of the source code.",
                        click_type=click.Choice(sorted(LANGUAGES)),
                    ),
                ] = None,
                target: Annotated[
                    Optional[str],
                    typer.Option(
                        "--target", "-t", help="Target file or string to evaluate."
                    ),
                ] = None,
                json_file_name: Annotated[
                    Optional[str],
                    typer.Option(
                        "--json",
                        "-j",
                        help="Json file to extract pairs from \
                            (if set ignores --target)",
                    ),
                ] = None,
                target_key: Annotated[
                    str,
                    typer.Option(
                        "--target-key",
                        "-tk",
                        help="json key to extract list of target strings",
                    ),
                ] = "target",
                splitting_method: Annotated[
                    str,
                    typer.Option(
                        "--method",
                        "-m",
                        click_type=click.Choice(SPLITTING_METHODS.keys()),
                        help="Method to use for pairing\
                              segments of target and reference files.",
                    ),
                ] = "file",
                llm_name: Annotated[
                    str,
                    typer.Option(
                        "--llm",
                        "-L",
                        help="The custom name of the model set with 'janus llm add'.",
                    ),
                ] = "gpt-4o",
                progress: Annotated[
                    bool,
                    typer.Option(
                        "--progress",
                        "-p",
                        help="Whether to display a progress bar.",
                        is_flag=True,
                    ),
                ] = False,
                use_strings: Annotated[
                    bool,
                    typer.Option(
                        "--string",
                        "-S",
                        help="Indicate that the target and reference are strings",
                        is_flag=True,
                    ),
                ] = False,
                *args,
                **kwargs,
            ):
                llm, _, token_limit, model_cost = load_model(llm_name)
                if json_file_name is not None:
                    with open(json_file_name, "r") as f:
                        json_obj = json.load(f)
                    strings = {}
                    for key in json_obj:
                        doc = json_obj[key]
                        experiments = doc["experiments"]
                        for model_key in experiments:
                            model_dict = experiments[model_key]
                            if not isinstance(model_dict, dict):
                                continue
                            if target_key not in model_dict:
                                continue
                            if model_key not in strings:
                                strings[model_key] = {}
                            for k in model_dict[target_key]:
                                strings[model_key][k] = model_dict[target_key][k]
                        # strings += list(json_obj[key][target_key].values())
                elif target is not None:
                    if use_strings:
                        target_contents = target
                    else:
                        with open(target, "r") as f:
                            target_contents = f.read()

                    strings = SPLITTING_METHODS[splitting_method](
                        target_contents,
                        target_file=target if not use_strings else None,
                        out_file=out_file,
                        lang=language,
                        llm=llm,
                        token_limit=token_limit,
                        model_cost=model_cost,
                    )
                else:
                    raise ValueError(
                        "Error: must specify either json file or target file/string"
                    )
                if isinstance(strings, dict):
                    out = {}
                    for k in strings:
                        out[k] = apply_function_strings(
                            strings[k],
                            function,
                            progress,
                            language,
                            llm,
                            token_limit,
                            model_cost,
                            *args,
                            **kwargs,
                        )
                else:
                    out = apply_function_strings(
                        strings,
                        function,
                        progress,
                        language,
                        llm,
                        token_limit,
                        model_cost,
                        *args,
                        **kwargs,
                    )
                out_file = Path(out_file)
                out_file.parent.mkdir(parents=True, exist_ok=True)
                with open(out_file, "w") as f:
                    json.dump(out, f)
                    log.info(f"Saved results to file: {out_file}")

            sig1 = inspect.signature(function)
            sig2 = inspect.signature(func)
            func.__signature__ = sig2.replace(
                parameters=tuple(
                    list(sig2.parameters.values())[:9]
                    + list(sig1.parameters.values())[1:-1]
                )
            )
        if name is None:
            func.__name__ = function.__name__
        else:
            func.__name__ = name
        if help is None:
            func = evaluate.command()(func)
        else:
            func = evaluate.command(help=help)(func)
        return function

    return decorator


def apply_function_pairs(
    pairs,
    function,
    progress,
    language,
    llm,
    token_limit,
    model_cost,
    *args,
    **kwargs,
):
    out = []
    pair_keys = None
    if isinstance(pairs, dict):
        pair_keys = list(pairs.keys())
        pair_values = list(pairs.values())
    else:
        pair_values = pairs
    if progress:
        loop = track(pair_values, description="Evaluating pairs")
    else:
        loop = pair_values
    for src, cmp in loop:
        if not (isinstance(src, str) and isinstance(cmp, str)):
            out.append(False)
        else:
            out.append(
                function(
                    src,
                    cmp,
                    *args,
                    **kwargs,
                    language=language,
                    llm=llm,
                    token_limit=token_limit,
                    model_cost=model_cost,
                )
            )
    if pair_keys is not None:
        return {k: v for k, v in zip(pair_keys, out)}
    return out


def apply_function_strings(
    strings, function, progress, language, llm, token_limit, model_cost, *args, **kwargs
):
    out = []
    string_keys = None
    if isinstance(strings, dict):
        string_keys = list(strings.keys())
        string_values = list(strings.values())
    else:
        string_values = strings
    if progress:
        loop = track(string_values, description="Evaluating strings")
    else:
        loop = string_values
    for string in loop:
        if not isinstance(string, str):
            out.append(False)
        else:
            out.append(
                function(
                    string,
                    *args,
                    **kwargs,
                    language=language,
                    llm=llm,
                    token_limit=token_limit,
                    model_cost=model_cost,
                )
            )
    if string_keys is not None:
        return {k: v for k, v in zip(string_keys, out)}
    return out



# from https://stackoverflow.com/a/76349078
# configure our tests by default not to run slow tests
#
# to run slow tests along with the other tests, use:
#   pytest -m slow

allowed_markers = (
    "filterwarnings",
    "parametrize",
    "skip",
    "skipif",
    "usefixtures",
    "xfail",
)


def pytest_collection_modifyitems(config, items):
    for item in items:
        if all(False for x in item.iter_markers() if x.name not in allowed_markers):
            item.add_marker("always_run")
    if markexpr := config.getoption("markexpr", "False"):
        config.option.markexpr = f"always_run or ({markexpr})"
        print("[Including tests marked as slow]")
    else:
        config.option.markexpr = "always_run"
        print("[Use -m slow to include tests marked as slow]")


def pytest_configure(config):
    config.addinivalue_line("markers", "always_run: mark test as always run")
    config.addinivalue_line("markers", "slow: mark test as slow")



import unittest
from pathlib import Path
from unittest.mock import patch

from janus.embedding.database import ChromaEmbeddingDatabase, uri_to_path


class TestDatabase(unittest.TestCase):
    def test_uri_to_path(self):
        uri = (Path.home().expanduser() / "Documents" / "testfile.txt").as_uri()
        expected_path = Path.home().expanduser() / "Documents" / "testfile.txt"
        self.assertEqual(uri_to_path(uri), expected_path)

    @patch("chromadb.PersistentClient", autospec=True)
    def test_ChromaEmbeddingDatabase(self, mock_client):
        # Test with default path
        _ = ChromaEmbeddingDatabase()
        mock_client.assert_called_once()

        # Test with custom path
        custom_path = "/custom/path/to/chroma-data"
        _ = ChromaEmbeddingDatabase(custom_path)
        mock_client.assert_called()

        # Test with URL
        url = "http://example.com/chroma-data"
        _ = ChromaEmbeddingDatabase(url)
        mock_client.assert_called()


if __name__ == "__main__":
    unittest.main()



import threading
from contextlib import contextmanager
from contextvars import ContextVar
from typing import Any, Generator

from langchain_core.callbacks import BaseCallbackHandler
from langchain_core.messages import AIMessage
from langchain_core.outputs import ChatGeneration, LLMResult
from langchain_core.tracers.context import register_configure_hook

from janus.utils.logger import create_logger

log = create_logger(__name__)


# Updated 2024-06-21
COST_PER_1K_TOKENS: dict[str, dict[str, float]] = {
    "gpt-3.5-turbo-0125": {"input": 0.0005, "output": 0.0015},
    "gpt-4-1106-preview": {"input": 0.01, "output": 0.03},
    "gpt-4-0125-preview": {"input": 0.01, "output": 0.03},
    "gpt-4-0613": {"input": 0.03, "output": 0.06},
    "gpt-4o-2024-05-13": {"input": 0.005, "output": 0.015},
    "anthropic.claude-v2": {"input": 0.008, "output": 0.024},
    "anthropic.claude-instant-v1": {"input": 0.0008, "output": 0.0024},
    "anthropic.claude-3-haiku-20240307-v1:0": {"input": 0.00025, "output": 0.00125},
    "anthropic.claude-3-sonnet-20240229-v1:0": {"input": 0.003, "output": 0.015},
    "meta.llama2-13b-chat-v1": {"input": 0.00075, "output": 0.001},
    "meta.llama2-70b-chat-v1": {"input": 0.00195, "output": 0.00256},
    "meta.llama2-13b-v1": {"input": 0.0, "output": 0.0},
    "meta.llama2-70b-v1": {"input": 0.00265, "output": 0.0035},
    "meta.llama3-8b-instruct-v1:0": {"input": 0.0003, "output": 0.0006},
    "meta.llama3-70b-instruct-v1:0": {"input": 0.00265, "output": 0.0035},
    "amazon.titan-text-lite-v1": {"input": 0.00015, "output": 0.0002},
    "amazon.titan-text-express-v1": {"input": 0.0002, "output": 0.0006},
    "ai21.j2-mid-v1": {"input": 0.0125, "output": 0.0125},
    "ai21.j2-ultra-v1": {"input": 0.0188, "output": 0.0188},
    "cohere.command-r-plus-v1:0": {"input": 0.003, "output": 0.015},
    "mistral.mistral-7b-instruct-v0:2": {"input": 0.00015, "output": 0.0002},
    "mistral.mixtral-8x7b-instruct-v0:1": {"input": 0.00045, "output": 0.0007},
    "mistral.mistral-large-2402-v1:0": {"input": 0.004, "output": 0.012},
}


def _get_token_cost(
    prompt_tokens: int, completion_tokens: int, model_id: str | None
) -> float:
    """Get the cost of tokens according to model ID"""
    if model_id not in COST_PER_1K_TOKENS:
        raise ValueError(
            f"Unknown model: {model_id}. Please provide a valid model name."
            f" Known models are: {', '.join(COST_PER_1K_TOKENS.keys())}"
        )
    model_cost = COST_PER_1K_TOKENS[model_id]
    input_cost = (prompt_tokens / 1000.0) * model_cost["input"]
    output_cost = (completion_tokens / 1000.0) * model_cost["output"]
    return input_cost + output_cost


class TokenUsageCallbackHandler(BaseCallbackHandler):
    """Callback Handler that tracks metadata on model cost, retries, etc.
    Based on https://github.com/langchain-ai/langchain/blob/master/libs
                /community/langchain_community/callbacks/openai_info.py
    """

    total_tokens: int = 0
    prompt_tokens: int = 0
    completion_tokens: int = 0
    successful_requests: int = 0
    total_cost: float = 0.0

    def __init__(self) -> None:
        super().__init__()
        self._lock = threading.Lock()

    def __repr__(self) -> str:
        return (
            f"Tokens Used: {self.total_tokens}\n"
            f"\tPrompt Tokens: {self.prompt_tokens}\n"
            f"\tCompletion Tokens: {self.completion_tokens}\n"
            f"Successful Requests: {self.successful_requests}\n"
            f"Total Cost (USD): ${self.total_cost}"
        )

    @property
    def always_verbose(self) -> bool:
        """Whether to call verbose callbacks even if verbose is False."""
        return True

    def on_chat_model_start(self, *args, **kwargs):
        pass

    def on_llm_start(
        self, serialized: dict[str, Any], prompts: list[str], **kwargs: Any
    ) -> None:
        """Print out the prompts."""
        pass

    def on_llm_new_token(self, token: str, **kwargs: Any) -> None:
        """Print out the token."""
        pass

    def on_llm_end(self, response: LLMResult, **kwargs: Any) -> None:
        """Collect token usage."""
        # Check for usage_metadata (langchain-core >= 0.2.2)
        try:
            generation = response.generations[0][0]
        except IndexError:
            generation = None

        model_id = ""
        usage_metadata = None
        if hasattr(response, "llm_output") and response.llm_output is not None:
            model_id = response.llm_output.get("model_id", model_id)
            model_id = response.llm_output.get("model_name", model_id)
            usage_metadata = response.llm_output.get("usage", usage_metadata)
            usage_metadata = response.llm_output.get("token_usage", usage_metadata)
        elif isinstance(generation, ChatGeneration):
            if hasattr(generation, "response_metadata"):
                model_id = generation.response_metadata.get("model_id", model_id)
                model_id = generation.response_metadata.get("model_name", model_id)
                usage_metadata = generation.response_metadata.get("usage", usage_metadata)
            elif hasattr(generation, "message"):
                if isinstance(generation.message, AIMessage):
                    usage_metadata = generation.message.usage_metadata

        completion_tokens = 0
        prompt_tokens = 0
        total_tokens = 0
        if usage_metadata:
            prompt_tokens = usage_metadata.get("prompt_tokens", prompt_tokens)
            prompt_tokens = usage_metadata.get("input_tokens", prompt_tokens)
            completion_tokens = usage_metadata.get("completion_tokens", completion_tokens)
            completion_tokens = usage_metadata.get("output_tokens", completion_tokens)
            total_tokens = usage_metadata.get("total_tokens", total_tokens)
        else:
            with self._lock:
                self.successful_requests += 1
            return None

        total_cost = _get_token_cost(
            prompt_tokens=prompt_tokens,
            completion_tokens=completion_tokens,
            model_id=model_id,
        )

        # update shared state behind lock
        with self._lock:
            self.total_cost += total_cost
            self.total_tokens += total_tokens
            self.prompt_tokens += prompt_tokens
            self.completion_tokens += completion_tokens
            self.successful_requests += 1

    def __copy__(self) -> "TokenUsageCallbackHandler":
        """Return a copy of the callback handler."""
        return self

    def __deepcopy__(self, memo: Any) -> "TokenUsageCallbackHandler":
        """Return a deep copy of the callback handler."""
        return self


token_usage_callback_var: ContextVar[TokenUsageCallbackHandler | None] = ContextVar(
    "token_usage_callback_var", default=None
)
register_configure_hook(token_usage_callback_var, True)


@contextmanager
def get_model_callback() -> Generator[TokenUsageCallbackHandler, None, None]:
    cb = TokenUsageCallbackHandler()
    token_usage_callback_var.set(cb)
    yield cb
    token_usage_callback_var.set(None)



from pathlib import Path
from typing import Any

import click
import typer
from langchain_core.exceptions import OutputParserException
from langchain_core.output_parsers import BaseOutputParser, JsonOutputParser
from langchain_core.prompts import PromptTemplate
from langchain_core.pydantic_v1 import BaseModel, Field
from typing_extensions import Annotated

from janus.metrics.metric import metric


class LLMMetricOutput(BaseModel):
    """The output of an LLM evaluation metric."""

    thought: str = Field(
        ...,
        description=(
            "The thought process that you took to reach your value determination."
        ),
    )
    value: str | float | int = Field(
        ..., description="The value of the metric described in the prompt."
    )


def load_prompt(path: Path, language: str, parser: BaseOutputParser) -> PromptTemplate:
    """Load a default prompt from a file.

    Arguments:
        path: The path to the file.
        language: The language of the prompt.
        pydantic_model: The Pydantic model to use for parsing the output.

    Returns:
        The prompt text.
    """
    if not path.exists():
        raise FileNotFoundError(f"File not found: {path}")
    prompt = PromptTemplate.from_template(
        path.read_text(),
        template_format="f-string",
        partial_variables={
            "language": language,
            "format_instructions": parser.get_format_instructions(),
        },
    )
    return prompt


def evaluate(
    target: str,
    language: str,
    model: str,
    prompt_path: Path,
    reference: str | None = None,
):
    """Calculate the LLM self evaluation score.

    Arguments:
        target: The target text.
        language: The language that the target code is written in.
        prompt_path: The filepath of the prompt text
        reference: The reference text.

    Returns:
        The LLM Evaluation score.
    """
    parser = JsonOutputParser(pydantic_object=LLMMetricOutput)
    prompt = load_prompt(prompt_path, language, parser)
    chain = prompt | model | parser
    try:
        output = (
            chain.invoke(dict(target=target, reference=reference))
            if reference
            else chain.invoke(dict(target=target))
        )
        return output["value"]
    except OutputParserException:
        return False


@metric(use_reference=False, name="llm", help="LLM self-evaluation on a target file")
def llm_evaluate_option(
    target: str,
    metric: Annotated[
        str,
        typer.Option(
            "--metric",
            "-m",
            help=("The pre-defined metric to use for evaluation."),
            click_type=click.Choice(
                [
                    "quality",
                    "clarity",
                    "faithfulness",
                    "completeness",
                    "hallucination",
                    "readability",
                    "usefulness",
                ]
            ),
        ),
    ] = "quality",
    prompt: Annotated[
        str,
        None,
        typer.Option(
            "--prompt",
            "-P",
            help=("A custom prompt in a .txt file to use for evaluation."),
        ),
    ] = None,
    num_eval: Annotated[
        int,
        typer.Option(
            "-n",
            "--num-eval",
            help="Number of times to run the evaluation",
        ),
    ] = 1,
    **kwargs,
) -> Any:
    """CLI option to calculate the LLM self evaluation score.

    Arguments:
        target: The target text.
        reference: The reference text.
        metric: The pre-defined metric to use for evaluation.
        prompt: The prompt text.

    Returns:
        The LLM Evaluation score.
    """
    prompt_path: Path = (
        Path(prompt) if prompt else Path(__file__).parent / "prompts" / f"{metric}.txt"
    )
    if num_eval == 1:
        return evaluate(target, kwargs["language"], kwargs["llm"], prompt_path)
    else:
        return [
            evaluate(target, kwargs["language"], kwargs["llm"], prompt_path)
            for _ in range(num_eval)
        ]


@metric(name="llm-ref", help="LLM self-evaluation on a target file and a reference file")
def llm_evaluate_ref_option(
    target: str,
    reference: str,
    metric: Annotated[
        str,
        typer.Option(
            "--metric",
            "-m",
            help=("The pre-defined metric to use for evaluation."),
            click_type=click.Choice(["faithfulness"]),
        ),
    ] = "faithfulness",
    prompt: Annotated[
        str,
        None,
        typer.Option(
            "--prompt",
            "-P",
            help=("A custom prompt in a .txt file to use for evaluation."),
        ),
    ] = None,
    num_eval: Annotated[
        int,
        typer.Option(
            "-n",
            "--num-eval",
            help="Number of times to run evaluation for pair",
        ),
    ] = 1,
    **kwargs,
) -> Any:
    """CLI option to calculate the LLM self evaluation score, for evaluations which
    require a reference file (e.g. faithfulness)

    Arguments:
        target: The target text.
        reference: The reference text.
        metric: The pre-defined metric to use for evaluation.
        prompt: The prompt text.

    Returns:
        The LLM Evaluation score.
    """
    prompt_path: Path = (
        Path(prompt) if prompt else Path(__file__).parent / "prompts" / f"{metric}.txt"
    )
    if num_eval == 1:
        return evaluate(target, kwargs["language"], kwargs["llm"], prompt_path, reference)
    else:
        return [
            evaluate(target, kwargs["language"], kwargs["llm"], prompt_path, reference)
            for _ in range(num_eval)
        ]



import math

from langchain.output_parsers import RetryWithErrorOutputParser
from langchain_core.exceptions import OutputParserException
from langchain_core.runnables import RunnableLambda, RunnableParallel

from janus.converter.converter import run_if_changed
from janus.converter.document import Documenter
from janus.language.block import TranslatedCodeBlock
from janus.llm.models_info import MODEL_PROMPT_ENGINES
from janus.parsers.refiner_parser import RefinerParser
from janus.parsers.uml import UMLSyntaxParser
from janus.utils.logger import create_logger

log = create_logger(__name__)


class DiagramGenerator(Documenter):
    """DiagramGenerator

    A class that translates code from one programming language to a set of diagrams.
    """

    def __init__(
        self,
        diagram_type="Activity",
        add_documentation=False,
        **kwargs,
    ) -> None:
        """Initialize the DiagramGenerator class

        Arguments:
            model: The LLM to use for translation. If an OpenAI model, the
                `OPENAI_API_KEY` environment variable must be set and the
                `OPENAI_ORG_ID` environment variable should be set if needed.
            model_arguments: Additional arguments to pass to the LLM constructor.
            source_language: The source programming language.
            max_prompts: The maximum number of prompts to try before giving up.
            db_path: path to chroma database
            db_config: database configuraiton
            diagram_type: type of PLANTUML diagram to generate
        """
        super().__init__(**kwargs)
        self._diagram_type = diagram_type
        self._add_documentation = add_documentation
        self._documenter = None
        self._diagram_parser = UMLSyntaxParser(language="plantuml")
        if add_documentation:
            self._diagram_prompt_template_name = "diagram_with_documentation"
        else:
            self._diagram_prompt_template_name = "diagram"
        self._load_diagram_prompt_engine()

    def _run_chain(self, block: TranslatedCodeBlock) -> str:
        self._parser.set_reference(block.original)
        n1 = round(self.max_prompts ** (1 / 3))

        # Retries with the input, output, and error
        n2 = round((self.max_prompts // n1) ** (1 / 2))

        # Retries with just the input
        n3 = math.ceil(self.max_prompts / (n1 * n2))

        if self._add_documentation:
            documentation_text = super()._run_chain(block)
            refine_output = RefinerParser(
                parser=self._diagram_parser,
                initial_prompt=self._diagram_prompt.format(
                    **{
                        "SOURCE_CODE": block.original.text,
                        "DOCUMENTATION": documentation_text,
                        "DIAGRAM_TYPE": self._diagram_type,
                    }
                ),
                refiner=self._refiner,
                max_retries=n1,
                llm=self._llm,
            )
        else:
            refine_output = RefinerParser(
                parser=self._diagram_parser,
                initial_prompt=self._diagram_prompt.format(
                    **{
                        "SOURCE_CODE": block.original.text,
                        "DIAGRAM_TYPE": self._diagram_type,
                    }
                ),
                refiner=self._refiner,
                max_retries=n1,
                llm=self._llm,
            )
        retry = RetryWithErrorOutputParser.from_llm(
            llm=self._llm,
            parser=refine_output,
            max_retries=n2,
        )
        completion_chain = self._prompt | self._llm
        chain = RunnableParallel(
            completion=completion_chain, prompt_value=self._diagram_prompt
        ) | RunnableLambda(lambda x: retry.parse_with_prompt(**x))
        for _ in range(n3):
            try:
                if self._add_documentation:
                    return chain.invoke(
                        {
                            "SOURCE_CODE": block.original.text,
                            "DOCUMENTATION": documentation_text,
                            "DIAGRAM_TYPE": self._diagram_type,
                        }
                    )
                else:
                    return chain.invoke(
                        {
                            "SOURCE_CODE": block.original.text,
                            "DIAGRAM_TYPE": self._diagram_type,
                        }
                    )
            except OutputParserException:
                pass

        raise OutputParserException(f"Failed to parse after {n1*n2*n3} retries")

    @run_if_changed(
        "_diagram_prompt_template_name",
        "_source_language",
    )
    def _load_diagram_prompt_engine(self) -> None:
        """Load the prompt engine according to this instance's attributes.

        If the relevant fields have not been changed since the last time this method was
        called, nothing happens.
        """
        self._diagram_prompt_engine = MODEL_PROMPT_ENGINES[self._model_name](
            source_language=self._source_language,
            target_language="text",
            target_version=None,
            prompt_template=self._diagram_prompt_template_name,
        )
        self._diagram_prompt = self._diagram_prompt_engine.prompt



import json
import os
from pathlib import Path
from typing import Any, Callable

from dotenv import load_dotenv
from langchain_community.llms import HuggingFaceTextGenInference
from langchain_core.language_models import BaseLanguageModel
from langchain_openai import ChatOpenAI

from janus.llm.model_callbacks import COST_PER_1K_TOKENS
from janus.prompts.prompt import (
    ChatGptPromptEngine,
    ClaudePromptEngine,
    CoherePromptEngine,
    Llama2PromptEngine,
    Llama3PromptEngine,
    MistralPromptEngine,
    PromptEngine,
    TitanPromptEngine,
)
from janus.utils.logger import create_logger

log = create_logger(__name__)

try:
    from langchain_community.chat_models import BedrockChat
    from langchain_community.llms.bedrock import Bedrock
except ImportError:
    log.warning(
        "Could not import LangChain's Bedrock Client. If you would like to use Bedrock "
        "models, please install LangChain's Bedrock Client by running 'pip install "
        "janus-llm[bedrock]' or poetry install -E bedrock."
    )

try:
    from langchain_community.llms.huggingface_pipeline import HuggingFacePipeline
except ImportError:
    log.warning(
        "Could not import LangChain's HuggingFace Pipeline Client. If you would like to "
        "use HuggingFace models, please install LangChain's HuggingFace Pipeline Client "
        "by running 'pip install janus-llm[hf-local]' or poetry install -E hf-local."
    )


load_dotenv()

openai_model_reroutes = {
    "gpt-4o": "gpt-4o-2024-05-13",
    "gpt-4o-mini": "gpt-4o-mini",
    "gpt-4": "gpt-4-0613",
    "gpt-4-turbo": "gpt-4-turbo-2024-04-09",
    "gpt-4-turbo-preview": "gpt-4-0125-preview",
    "gpt-3.5-turbo": "gpt-3.5-turbo-0125",
    "gpt-3.5-turbo-16k": "gpt-3.5-turbo-0125",
}

openai_models = [
    "gpt-4o",
    "gpt-4o-mini",
    "gpt-4",
    "gpt-4-turbo",
    "gpt-4-turbo-preview",
    "gpt-3.5-turbo",
    "gpt-3.5-turbo-16k",
]
claude_models = [
    "bedrock-claude-v2",
    "bedrock-claude-instant-v1",
    "bedrock-claude-haiku",
    "bedrock-claude-sonnet",
]
llama2_models = [
    "bedrock-llama2-70b",
    "bedrock-llama2-70b-chat",
    "bedrock-llama2-13b",
    "bedrock-llama2-13b-chat",
]
llama3_models = [
    "bedrock-llama3-8b-instruct",
    "bedrock-llama3-70b-instruct",
]
titan_models = [
    "bedrock-titan-text-lite",
    "bedrock-titan-text-express",
    "bedrock-jurassic-2-mid",
    "bedrock-jurassic-2-ultra",
]
cohere_models = [
    "bedrock-command-r-plus",
]
mistral_models = [
    "bedrock-mistral-7b-instruct",
    "bedrock-mistral-large",
    "bedrock-mixtral",
]
bedrock_models = [
    *claude_models,
    *llama2_models,
    *llama3_models,
    *titan_models,
    *cohere_models,
    *mistral_models,
]
all_models = [*openai_models, *bedrock_models]

MODEL_TYPE_CONSTRUCTORS: dict[str, Callable[[Any], BaseLanguageModel]] = {
    "OpenAI": ChatOpenAI,
    "HuggingFace": HuggingFaceTextGenInference,
}

try:
    MODEL_TYPE_CONSTRUCTORS.update(
        {
            "HuggingFaceLocal": HuggingFacePipeline.from_model_id,
            "Bedrock": Bedrock,
            "BedrockChat": BedrockChat,
        }
    )
except NameError:
    pass


MODEL_PROMPT_ENGINES: dict[str, Callable[..., PromptEngine]] = {
    **{m: ChatGptPromptEngine for m in openai_models},
    **{m: ClaudePromptEngine for m in claude_models},
    **{m: Llama2PromptEngine for m in llama2_models},
    **{m: Llama3PromptEngine for m in llama3_models},
    **{m: TitanPromptEngine for m in titan_models},
    **{m: CoherePromptEngine for m in cohere_models},
    **{m: MistralPromptEngine for m in mistral_models},
}

_open_ai_defaults: dict[str, str] = {
    "openai_api_key": os.getenv("OPENAI_API_KEY"),
    "openai_organization": os.getenv("OPENAI_ORG_ID"),
}

MODEL_ID_TO_LONG_ID = {
    **{m: mr for m, mr in openai_model_reroutes.items()},
    "bedrock-claude-v2": "anthropic.claude-v2",
    "bedrock-claude-instant-v1": "anthropic.claude-instant-v1",
    "bedrock-claude-haiku": "anthropic.claude-3-haiku-20240307-v1:0",
    "bedrock-claude-sonnet": "anthropic.claude-3-sonnet-20240229-v1:0",
    "bedrock-llama2-70b": "meta.llama2-70b-v1",
    "bedrock-llama2-70b-chat": "meta.llama2-70b-chat-v1",
    "bedrock-llama2-13b": "meta.llama2-13b-chat-v1",
    "bedrock-llama2-13b-chat": "meta.llama2-13b-v1",
    "bedrock-llama3-8b-instruct": "meta.llama3-8b-instruct-v1:0",
    "bedrock-llama3-70b-instruct": "meta.llama3-70b-instruct-v1:0",
    "bedrock-titan-text-lite": "amazon.titan-text-lite-v1",
    "bedrock-titan-text-express": "amazon.titan-text-express-v1",
    "bedrock-jurassic-2-mid": "ai21.j2-mid-v1",
    "bedrock-jurassic-2-ultra": "ai21.j2-ultra-v1",
    "bedrock-command-r-plus": "cohere.command-r-plus-v1:0",
    "bedrock-mixtral": "mistral.mixtral-8x7b-instruct-v0:1",
    "bedrock-mistral-7b-instruct": "mistral.mistral-7b-instruct-v0:2",
    "bedrock-mistral-large": "mistral.mistral-large-2402-v1:0",
}

MODEL_DEFAULT_ARGUMENTS: dict[str, dict[str, str]] = {
    k: (dict(model_name=k) if k in openai_models else dict(model_id=v))
    for k, v in MODEL_ID_TO_LONG_ID.items()
}

DEFAULT_MODELS = list(MODEL_DEFAULT_ARGUMENTS.keys())

MODEL_CONFIG_DIR = Path.home().expanduser() / ".janus" / "llm"

MODEL_TYPES: dict[str, PromptEngine] = {
    **{m: "OpenAI" for m in openai_models},
    **{m: "BedrockChat" for m in bedrock_models},
}

TOKEN_LIMITS: dict[str, int] = {
    "gpt-4-32k": 32_768,
    "gpt-4-0613": 8192,
    "gpt-4-1106-preview": 128_000,
    "gpt-4-0125-preview": 128_000,
    "gpt-4o-2024-05-13": 128_000,
    "gpt-3.5-turbo-0125": 16_384,
    "text-embedding-ada-002": 8191,
    "gpt4all": 16_384,
    "anthropic.claude-v2": 100_000,
    "anthropic.claude-instant-v1": 100_000,
    "anthropic.claude-3-haiku-20240307-v1:0": 248_000,
    "anthropic.claude-3-sonnet-20240229-v1:0": 248_000,
    "meta.llama2-70b-v1": 4096,
    "meta.llama2-70b-chat-v1": 4096,
    "meta.llama2-13b-chat-v1": 4096,
    "meta.llama2-13b-v1": 4096,
    "meta.llama3-8b-instruct-v1:0": 8000,
    "meta.llama3-70b-instruct-v1:0": 8000,
    "amazon.titan-text-lite-v1": 4096,
    "amazon.titan-text-express-v1": 8192,
    "ai21.j2-mid-v1": 8192,
    "ai21.j2-ultra-v1": 8192,
    "cohere.command-r-plus-v1:0": 128_000,
    "mistral.mixtral-8x7b-instruct-v0:1": 32_000,
    "mistral.mistral-7b-instruct-v0:2": 32_000,
    "mistral.mistral-large-2402-v1:0": 32_000,
}


def get_available_model_names() -> list[str]:
    avaialable_models = []
    for file in MODEL_CONFIG_DIR.iterdir():
        if file.is_file():
            avaialable_models.append(MODEL_CONFIG_DIR.stem)
    return avaialable_models


def load_model(user_model_name: str) -> tuple[BaseLanguageModel, int, dict[str, float]]:
    if not MODEL_CONFIG_DIR.exists():
        MODEL_CONFIG_DIR.mkdir(parents=True)
    model_config_file = MODEL_CONFIG_DIR / f"{user_model_name}.json"
    if not model_config_file.exists():
        log.warning(
            f"Model {user_model_name} not found in user-defined models, searching "
            f"default models for {user_model_name}."
        )
        model_id = user_model_name
        if user_model_name not in DEFAULT_MODELS:
            message = (
                f"Model {user_model_name} not found in default models. Make sure to run "
                "`janus llm add` first."
            )
            log.error(message)
            raise ValueError(message)
        model_config = {
            "model_type": MODEL_TYPES[model_id],
            "model_id": model_id,
            "model_args": MODEL_DEFAULT_ARGUMENTS[model_id],
            "token_limit": TOKEN_LIMITS.get(MODEL_ID_TO_LONG_ID[model_id], 4096),
            "model_cost": COST_PER_1K_TOKENS.get(
                MODEL_ID_TO_LONG_ID[model_id], {"input": 0, "output": 0}
            ),
        }
        with open(model_config_file, "w") as f:
            json.dump(model_config, f)
    else:
        with open(model_config_file, "r") as f:
            model_config = json.load(f)
    model_constructor = MODEL_TYPE_CONSTRUCTORS[model_config["model_type"]]
    model_args = model_config["model_args"]
    if model_config["model_type"] == "OpenAI":
        model_args.update(_open_ai_defaults)
    model = model_constructor(**model_args)
    return (
        model,
        model_config["model_id"],
        model_config["token_limit"],
        model_config["model_cost"],
    )



import unittest
from pathlib import Path

from janus.language.alc import AlcSplitter
from janus.language.combine import Combiner
from janus.llm import load_model


class TestAlcSplitter(unittest.TestCase):
    """Tests for the Splitter class."""

    def setUp(self):
        """Set up the tests."""
        model_name = "gpt-4o"
        llm, _, _, _ = load_model(model_name)
        self.splitter = AlcSplitter(model=llm)
        self.combiner = Combiner(language="ibmhlasm")
        self.test_file = Path("janus/language/alc/_tests/alc.asm")

    def test_split(self):
        """Test the split method."""
        tree_root = self.splitter.split(self.test_file)
        self.assertAlmostEqual(tree_root.n_descendents, 32, delta=5)
        self.assertLessEqual(tree_root.max_tokens, self.splitter.max_tokens)
        self.assertFalse(tree_root.complete)
        self.combiner.combine_children(tree_root)
        self.assertTrue(tree_root.complete)
        self.assertEqual(tree_root.complete_text, self.test_file.read_text())



from janus.language.block import CodeBlock, TranslatedCodeBlock
from janus.language.file import FileManager
from janus.utils.logger import create_logger

log = create_logger(__name__)


class Combiner(FileManager):
    """Combine files that have been split into functional blocks back into
    a single file.
    """

    @staticmethod
    def combine(root: CodeBlock) -> None:
        """Combine the given block with its children.

        Arguments:
            root: The functional code block to combine with its children.
        """
        Combiner.combine_children(root)
        root.omit_prefix = False

    @staticmethod
    def combine_children(block: CodeBlock) -> None:
        """Recursively combine block code with children code.

        Arguments:
            block: The functional code block to recursively replace children.
        """
        if block.complete:
            return

        if isinstance(block, TranslatedCodeBlock) and not block.translated:
            return

        children_complete = True
        for child in block.children:
            Combiner.combine_children(child)
            if not child.complete:
                children_complete = False

        # If input string is None, then this node consists exclusively of
        #  children with no other formatting. Simply concatenate the children.
        if block.text is None:
            children = sorted(block.children)
            block.text = "".join([c.complete_text for c in children])
            block.complete = children_complete
            return

        missing_children = []
        for child in block.children:
            if isinstance(block, TranslatedCodeBlock) and not child.translated:
                missing_children.append(child)
                continue

        if missing_children:
            missing_ids = [c.id for c in missing_children]
            log.warning(f"Some children not found in code: {missing_ids}")

        block.children = missing_children
        block.complete = children_complete and not missing_children


class JsonCombiner(Combiner):
    @staticmethod
    def combine(root: CodeBlock) -> None:
        """Combine the given block with its children.

        Arguments:
            root: The functional code block to combine with its children.
        """
        stack = [root]
        while stack:
            block = stack.pop()
            if block.children:
                stack.extend(block.children)
                block.affixes = ("", "")
            else:
                block.affixes = ("\n", "\n")
        super(JsonCombiner, JsonCombiner).combine(root)


class ChunkCombiner(Combiner):
    @staticmethod
    def combine(root: CodeBlock) -> None:
        """A combiner which doesn't actually combine the code blocks,
        instead preserving children

        Arguments:
            root: The functional code block to combine with its children.
        """
        return root



from typing import NewType, Tuple

from janus.utils.logger import create_logger

log = create_logger(__name__)


NodeType = NewType("NodeType", str)
NodeTypes = NewType("NodeTypes", Tuple[NodeType, ...])



import functools
import json
import math
import time
from pathlib import Path
from typing import Any

from langchain.output_parsers import RetryWithErrorOutputParser
from langchain_core.exceptions import OutputParserException
from langchain_core.language_models import BaseLanguageModel
from langchain_core.output_parsers import BaseOutputParser
from langchain_core.prompts import ChatPromptTemplate
from langchain_core.runnables import RunnableLambda, RunnableParallel
from openai import BadRequestError, RateLimitError
from pydantic import ValidationError

from janus.embedding.vectorize import ChromaDBVectorizer
from janus.language.block import CodeBlock, TranslatedCodeBlock
from janus.language.combine import Combiner
from janus.language.naive.registry import CUSTOM_SPLITTERS
from janus.language.splitter import (
    EmptyTreeError,
    FileSizeError,
    Splitter,
    TokenLimitError,
)
from janus.llm import load_model
from janus.llm.model_callbacks import get_model_callback
from janus.llm.models_info import MODEL_PROMPT_ENGINES
from janus.parsers.code_parser import GenericParser
from janus.parsers.refiner_parser import RefinerParser
from janus.refiners.refiner import BasicRefiner, Refiner
from janus.utils.enums import LANGUAGES
from janus.utils.logger import create_logger

log = create_logger(__name__)


def run_if_changed(*tracked_vars):
    """Wrapper to skip function calls if the given instance attributes haven't
    been updated. Requires the _changed_attrs set to exist, and the __setattr__
    method to be overridden to track parameter updates in _changed_attrs.
    """

    def wrapper(func):
        @functools.wraps(func)
        def wrapped(self, *args, **kwargs):
            # If there is overlap between the tracked variables and the changed
            #  ones, then call the function as normal
            if not tracked_vars or self._changed_attrs.intersection(tracked_vars):
                func(self, *args, **kwargs)

        return wrapped

    return wrapper


class Converter:
    """Parent class that converts code into something else.

    Children will determine what the code gets converted into. Whether that's translated
    into another language, into pseudocode, requirements, documentation, etc., or
    converted into embeddings
    """

    def __init__(
        self,
        model: str = "gpt-4o",
        model_arguments: dict[str, Any] = {},
        source_language: str = "fortran",
        max_prompts: int = 10,
        max_tokens: int | None = None,
        prompt_template: str = "simple",
        db_path: str | None = None,
        db_config: dict[str, Any] | None = None,
        protected_node_types: tuple[str, ...] = (),
        prune_node_types: tuple[str, ...] = (),
        splitter_type: str = "file",
        refiner_type: str = "basic",
    ) -> None:
        """Initialize a Converter instance.

        Arguments:
            source_language: The source programming language.
            parser_type: The type of parser to use for parsing the LLM output. Valid
                values are `"code"`, `"text"`, `"eval"`, and `None` (default). If `None`,
                the `Converter` assumes you won't be parsing an output (i.e., adding to an
                embedding DB).
            max_prompts: The maximum number of prompts to try before giving up.
            max_tokens: The maximum number of tokens to use in the LLM. If `None`, the
                converter will use half the model's token limit.
            prompt_template: The name of the prompt template to use.
            db_path: The path to the database to use for vectorization.
            db_config: The configuration for the database.
            protected_node_types: A set of node types that aren't to be merged.
            prune_node_types: A set of node types which should be pruned.
            splitter_type: The type of splitter to use. Valid values are `"file"`,
                `"tag"`, `"chunk"`, `"ast-strict"`, and `"ast-flex"`.
            refiner_type: The type of refiner to use. Valid values are `"basic"`.
        """
        self._changed_attrs: set = set()

        self.max_prompts: int = max_prompts
        self._max_tokens: int | None = max_tokens
        self.override_token_limit: bool = max_tokens is not None

        self._model_name: str
        self._model_id: str
        self._custom_model_arguments: dict[str, Any]

        self._source_language: str
        self._source_suffix: str

        self._target_language = "json"
        self._target_suffix = ".json"

        self._protected_node_types: tuple[str, ...] = ()
        self._prune_node_types: tuple[str, ...] = ()
        self._max_tokens: int | None = max_tokens
        self._prompt_template_name: str
        self._splitter_type: str
        self._db_path: str | None
        self._db_config: dict[str, Any] | None

        self._splitter: Splitter
        self._llm: BaseLanguageModel
        self._prompt: ChatPromptTemplate

        self._parser: BaseOutputParser = GenericParser()
        self._combiner: Combiner = Combiner()

        self._refiner_type: str
        self._refiner: Refiner

        self.set_splitter(splitter_type=splitter_type)
        self.set_refiner(refiner_type=refiner_type)
        self.set_model(model_name=model, **model_arguments)
        self.set_prompt(prompt_template=prompt_template)
        self.set_source_language(source_language)
        self.set_protected_node_types(protected_node_types)
        self.set_prune_node_types(prune_node_types)
        self.set_db_path(db_path=db_path)
        self.set_db_config(db_config=db_config)

        # Child class must call this. Should we enforce somehow?
        # self._load_parameters()

    def __setattr__(self, key: Any, value: Any) -> None:
        if hasattr(self, "_changed_attrs"):
            if not hasattr(self, key) or getattr(self, key) != value:
                self._changed_attrs.add(key)
        # Avoid infinite recursion
        elif key != "_changed_attrs":
            self._changed_attrs = set()
        super().__setattr__(key, value)

    def _load_parameters(self) -> None:
        self._load_model()
        self._load_prompt()
        self._load_splitter()
        self._load_vectorizer()
        self._load_refiner()
        self._changed_attrs.clear()

    def set_model(self, model_name: str, **custom_arguments: dict[str, Any]):
        """Validate and set the model name.

        The affected objects will not be updated until translate() is called.

        Arguments:
            model_name: The name of the model to use. Valid models are found in
                `janus.llm.models_info.MODEL_CONSTRUCTORS`.
            custom_arguments: Additional arguments to pass to the model constructor.
        """
        self._model_name = model_name
        self._custom_model_arguments = custom_arguments

    def set_prompt(self, prompt_template: str) -> None:
        """Validate and set the prompt template name.

        The affected objects will not be updated until translate() is called.

        Arguments:
            prompt_template: name of prompt template directory
                (see janus/prompts/templates) or path to a directory.
        """
        self._prompt_template_name = prompt_template

    def set_splitter(self, splitter_type: str) -> None:
        """Validate and set the prompt template name.

        The affected objects will not be updated until translate() is called.

        Arguments:
            prompt_template: name of prompt template directory
                (see janus/prompts/templates) or path to a directory.
        """
        self._splitter_type = splitter_type

    def set_refiner(self, refiner_type: str) -> None:
        """Validate and set the refiner name

        The affected objects will not be updated until translate is called

        Arguments:
            refiner_type: the name of the refiner to use
        """
        self._refiner_type = refiner_type

    def set_source_language(self, source_language: str) -> None:
        """Validate and set the source language.

        The affected objects will not be updated until _load_parameters() is called.

        Arguments:
            source_language: The source programming language.
        """
        source_language = source_language.lower()
        if source_language not in LANGUAGES:
            raise ValueError(
                f"Invalid source language: {source_language}. "
                "Valid source languages are found in `janus.utils.enums.LANGUAGES`."
            )

        ext = LANGUAGES[source_language]["suffix"]
        self._source_suffix = f".{ext}"
        self._source_language = source_language

    def set_protected_node_types(self, protected_node_types: tuple[str, ...]) -> None:
        """Set the protected (non-mergeable) node types. This will often be structures
        like functions, classes, or modules which you might want to keep separate

        The affected objects will not be updated until _load_parameters() is called.

        Arguments:
            protected_node_types: A set of node types that aren't to be merged
        """
        self._protected_node_types = tuple(set(protected_node_types or []))

    def set_prune_node_types(self, prune_node_types: tuple[str, ...]) -> None:
        """Set the node types to prune. This will often be structures
        like comments or whitespace which you might want to keep out of the LLM

        The affected objects will not be updated until _load_parameters() is called.

        Arguments:
            prune_node_types: A set of node types which should be pruned
        """
        self._prune_node_types = tuple(set(prune_node_types or []))

    def set_db_path(self, db_path: str | None) -> None:
        self._db_path = db_path

    def set_db_config(self, db_config: dict[str, Any] | None) -> None:
        self._db_config = db_config

    @run_if_changed(
        "_source_language",
        "_max_tokens",
        "_llm",
        "_protected_node_types",
        "_prune_node_types",
        "_custom_splitter",
    )
    def _load_splitter(self) -> None:
        """Load the splitter according to this instance's attributes.

        If the relevant fields have not been changed since the last time this method was
        called, nothing happens.
        """
        kwargs: dict[str, Any] = dict(
            language=self._source_language,
            max_tokens=self._max_tokens,
            model=self._llm,
            protected_node_types=self._protected_node_types,
            prune_node_types=self._prune_node_types,
        )

        if self._splitter_type == "tag":
            kwargs["tag"] = "<ITMOD_ALC_SPLIT>"  # Hardcoded for now

        self._splitter = CUSTOM_SPLITTERS[self._splitter_type](**kwargs)

    @run_if_changed("_refiner_type", "_model_name")
    def _load_refiner(self) -> None:
        """Load the refiner according to this instance's attributes.

        If the relevant fields have not been changed since the last time this method was
        called, nothing happens.
        """
        if self._refiner_type == "basic":
            self._refiner = BasicRefiner(
                "basic_refinement", self._model_name, self._source_language
            )
        else:
            raise ValueError(f"Error: unknown refiner type {self._refiner_type}")

    @run_if_changed("_model_name", "_custom_model_arguments")
    def _load_model(self) -> None:
        """Load the model according to this instance's attributes.

        If the relevant fields have not been changed since the last time this method was
        called, nothing happens.
        """

        # Get default arguments, set custom ones
        # model_arguments = deepcopy(MODEL_DEFAULT_ARGUMENTS[self._model_name])
        # model_arguments.update(self._custom_model_arguments)

        # Load the model
        self._llm, self._model_id, token_limit, self.model_cost = load_model(
            self._model_name
        )
        # Set the max_tokens to less than half the model's limit to allow for enough
        # tokens at output
        # Only modify max_tokens if it is not specified by user
        if not self.override_token_limit:
            self._max_tokens = int(token_limit // 2.5)

    @run_if_changed(
        "_prompt_template_name",
        "_source_language",
        "_model_name",
    )
    def _load_prompt(self) -> None:
        """Load the prompt according to this instance's attributes.

        If the relevant fields have not been changed since the last time this
        method was called, nothing happens.
        """
        prompt_engine = MODEL_PROMPT_ENGINES[self._model_id](
            source_language=self._source_language,
            prompt_template=self._prompt_template_name,
        )
        self._prompt = prompt_engine.prompt

    @run_if_changed("_db_path", "_db_config")
    def _load_vectorizer(self) -> None:
        if self._db_path is None or self._db_config is None:
            self._vectorizer = None
            return
        vectorizer_factory = ChromaDBVectorizer()
        self._vectorizer = vectorizer_factory.create_vectorizer(
            self._db_path, self._db_config
        )

    def translate(
        self,
        input_directory: str | Path,
        output_directory: str | Path | None = None,
        overwrite: bool = False,
        collection_name: str | None = None,
    ) -> None:
        """Convert code in the input directory from the source language to the target
        language, and write the resulting files to the output directory.

        Arguments:
            input_directory: The directory containing the code to translate.
            output_directory: The directory to write the translated code to.
            overwrite: Whether to overwrite existing files (vs skip them)
            collection_name: Collection to add to
        """
        # Convert paths to pathlib Paths if needed
        if isinstance(input_directory, str):
            input_directory = Path(input_directory)
        if isinstance(output_directory, str):
            output_directory = Path(output_directory)

        # Make sure the output directory exists
        if output_directory is not None and not output_directory.exists():
            output_directory.mkdir(parents=True)

        input_paths = [p for p in input_directory.rglob(f"**/*{self._source_suffix}")]

        log.info(f"Input directory: {input_directory.absolute()}")
        log.info(
            f"{self._source_language} '*{self._source_suffix}' files: "
            f"{len(input_paths)}"
        )
        log.info(
            "Other files (skipped): "
            f"{len(list(input_directory.iterdir())) - len(input_paths)}\n"
        )
        if output_directory is not None:
            output_paths = [
                output_directory
                / p.relative_to(input_directory).with_suffix(self._target_suffix)
                for p in input_paths
            ]
            in_out_pairs = list(zip(input_paths, output_paths))
            if not overwrite:
                n_files = len(in_out_pairs)
                in_out_pairs = [
                    (inp, outp) for inp, outp in in_out_pairs if not outp.exists()
                ]
                log.info(
                    f"Skipping {n_files - len(in_out_pairs)} existing "
                    f"'*{self._source_suffix}' files"
                )
        else:
            in_out_pairs = [(f, None) for f in input_paths]
        log.info(f"Translating {len(in_out_pairs)} '*{self._source_suffix}' files")

        # Loop through each input file, convert and save it
        total_cost = 0.0
        for in_path, out_path in in_out_pairs:
            # Translate the file, skip it if there's a rate limit error
            try:
                out_block = self.translate_file(in_path)
                total_cost += out_block.total_cost
            except RateLimitError:
                continue
            except OutputParserException as e:
                log.error(f"Skipping {in_path.name}, failed to parse output: {e}.")
                continue
            except BadRequestError as e:
                if str(e).startswith("Detected an error in the prompt"):
                    log.warning("Malformed input, skipping")
                    continue
                raise e
            except ValidationError as e:
                # Only allow ValidationError to pass if token limit is manually set
                if self.override_token_limit:
                    log.warning(
                        "Current file and manually set token "
                        "limit is too large for this model, skipping"
                    )
                    continue
                raise e
            except TokenLimitError:
                log.warning("Ran into irreducible node too large for context, skipping")
                continue
            except EmptyTreeError:
                log.warning(
                    f'Input file "{in_path.name}" has no nodes of interest, skipping'
                )
                continue
            except FileSizeError:
                log.warning("Current tile is too large for basic splitter, skipping")
                continue

            # Don't attempt to write files for which translation failed
            if not out_block.translated:
                continue

            if collection_name is not None:
                self._vectorizer.add_nodes_recursively(
                    out_block,
                    collection_name,
                    in_path.name,
                )

            # Make sure the tree's code has been consolidated at the top level
            #  before writing to file
            self._combiner.combine(out_block)
            if out_path is not None and (overwrite or not out_path.exists()):
                self._save_to_file(out_block, out_path)

        log.info(f"Total cost: ${total_cost:,.2f}")

    def translate_file(self, file: Path) -> TranslatedCodeBlock:
        """Translate a single file.

        Arguments:
            file: Input path to file

        Returns:
            A `TranslatedCodeBlock` object. This block does not have a path set, and its
            code is not guaranteed to be consolidated. To amend this, run
            `Combiner.combine_children` on the block.
        """
        self._load_parameters()
        filename = file.name

        input_block = self._split_file(file)
        t0 = time.time()
        output_block = self._iterative_translate(input_block)
        output_block.processing_time = time.time() - t0
        if output_block.translated:
            completeness = output_block.translation_completeness
            log.info(
                f"[{filename}] Translation complete\n"
                f"  {completeness:.2%} of input successfully translated\n"
                f"  Total cost: ${output_block.total_cost:,.2f}\n"
                f"  Total retries: {output_block.total_retries:,d}\n"
                f"  Output CodeBlock Structure:\n{input_block.tree_str()}\n"
            )

        else:
            log.error(
                f"[{filename}] Translation failed\n"
                f"  Total cost: ${output_block.total_cost:,.2f}\n"
                f"  Total retries: {output_block.total_retries:,d}\n"
            )
        return output_block

    def _iterative_translate(self, root: CodeBlock) -> TranslatedCodeBlock:
        """Translate the passed CodeBlock representing a full file.

        Arguments:
            root: A root block representing the top-level block of a file

        Returns:
            A `TranslatedCodeBlock`
        """
        translated_root = TranslatedCodeBlock(root, self._target_language)
        last_prog, prog_delta = 0, 0.1
        stack = [translated_root]
        while stack:
            translated_block = stack.pop()

            self._add_translation(translated_block)

            # If translating this block was unsuccessful, don't bother with its
            #  children (they wouldn't show up in the final text anyway)
            if not translated_block.translated:
                continue

            stack.extend(translated_block.children)

            progress = translated_root.translation_completeness
            if progress - last_prog > prog_delta:
                last_prog = int(progress / prog_delta) * prog_delta
                log.info(f"[{root.name}] progress: {progress:.2%}")

        return translated_root

    def _add_translation(self, block: TranslatedCodeBlock) -> None:
        """Given an "empty" `TranslatedCodeBlock`, translate the code represented in
        `block.original`, setting the relevant fields in the translated block. The
        `TranslatedCodeBlock` is updated in-pace, nothing is returned. Note that this
        translates *only* the code for this block, not its children.

        Arguments:
            block: An empty `TranslatedCodeBlock`
        """
        if block.translated:
            return

        if block.original.text is None:
            block.translated = True
            return

        if self._llm is None:
            message = (
                "Model not configured correctly, cannot translate. Try setting "
                "the model"
            )
            log.error(message)
            raise ValueError(message)

        log.debug(f"[{block.name}] Translating...")
        log.debug(f"[{block.name}] Input text:\n{block.original.text}")

        # Track the cost of translating this block
        #  TODO: If non-OpenAI models with prices are added, this will need
        #   to be updated.
        with get_model_callback() as cb:
            t0 = time.time()
            block.text = self._run_chain(block)
            block.processing_time = time.time() - t0
            block.cost = cb.total_cost
            block.retries = max(0, cb.successful_requests - 1)

        block.tokens = self._llm.get_num_tokens(block.text)
        block.translated = True

        log.debug(f"[{block.name}] Output code:\n{block.text}")

    def _split_file(self, file: Path) -> CodeBlock:
        filename = file.name
        log.info(f"[{filename}] Splitting file")
        root = self._splitter.split(file)
        log.info(
            f"[{filename}] File split into {root.n_descendents:,} blocks, "
            f"tree of height {root.height}"
        )
        log.info(f"[{filename}] Input CodeBlock Structure:\n{root.tree_str()}")
        return root

    def _run_chain(self, block: TranslatedCodeBlock) -> str:
        """Run the model with three nested error fixing schemes.
        First, try to fix simple formatting errors by giving the model just
        the output and the parsing error. After a number of attempts, try
        giving the model the output, the parsing error, and the original
        input. Again check/retry this output to solve for formatting errors.
        If we still haven't succeeded after several attempts, the model may
        be getting thrown off by a bad initial output; start from scratch
        and try again.

        The number of tries for each layer of this scheme is roughly equal
        to the cube root of self.max_retries, so the total calls to the
        LLM will be roughly as expected (up to sqrt(self.max_retries) over)
        """
        self._parser.set_reference(block.original)

        # Retries with just the output and the error
        n1 = round(self.max_prompts ** (1 / 3))

        # Retries with the input, output, and error
        n2 = round((self.max_prompts // n1) ** (1 / 2))

        # Retries with just the input
        n3 = math.ceil(self.max_prompts / (n1 * n2))

        refine_output = RefinerParser(
            parser=self._parser,
            initial_prompt=self._prompt.format(**{"SOURCE_CODE": block.original.text}),
            refiner=self._refiner,
            max_retries=n1,
            llm=self._llm,
        )
        retry = RetryWithErrorOutputParser.from_llm(
            llm=self._llm,
            parser=refine_output,
            max_retries=n2,
        )
        completion_chain = self._prompt | self._llm
        chain = RunnableParallel(
            completion=completion_chain, prompt_value=self._prompt
        ) | RunnableLambda(lambda x: retry.parse_with_prompt(**x))
        for _ in range(n3):
            try:
                return chain.invoke({"SOURCE_CODE": block.original.text})
            except OutputParserException:
                pass

        raise OutputParserException(f"Failed to parse after {n1*n2*n3} retries")

    def _get_output_obj(
        self, block: TranslatedCodeBlock
    ) -> dict[str, int | float | str | dict[str, str]]:
        output_str = self._parser.parse_combined_output(block.complete_text)

        output: str | dict[str, str]
        try:
            output = json.loads(output_str)
        except json.JSONDecodeError:
            output = output_str

        return dict(
            input=block.original.text,
            metadata=dict(
                retries=block.total_retries,
                cost=block.total_cost,
                processing_time=block.processing_time,
            ),
            output=output,
        )

    def _save_to_file(self, block: TranslatedCodeBlock, out_path: Path) -> None:
        """Save a file to disk.

        Arguments:
            block: The `TranslatedCodeBlock` to save to a file.
        """
        obj = self._get_output_obj(block)
        out_path.parent.mkdir(parents=True, exist_ok=True)
        out_path.write_text(json.dumps(obj, indent=2), encoding="utf-8")



import __main__ as ghidra_app
from ghidra.app.decompiler import DecompInterface


def decompile_function(decompiler, function):
    """Decompile a single function."""

    timeout = None
    # Try to decompile
    decompile_status = decompiler.decompileFunction(function, 0, timeout)
    # If it's successful...`
    if decompile_status and decompile_status.decompileCompleted():
        # Get and return the resulting C psuedocode
        decompiled_function = decompile_status.getDecompiledFunction()
        if decompiled_function:
            return decompiled_function.getC()


def decompile_program(decompiler):
    """Decompile each function in the current program."""

    # We'll decompilation of functions to this string as we go through each
    decompiled_buffer = ""
    all_functions = ghidra_app.currentProgram.getListing().getFunctions(True)

    # Set up a filter for imported library functions etc., which we presumably are not
    # interested in
    external_functions = (
        ghidra_app.currentProgram.getFunctionManager().getExternalFunctions()
    )
    external_function_names = []
    for function in external_functions:
        external_function_names.append(function.getName())

    # Iterate over all the non-imported functions, decompiling pushing each to the buffer
    for function in all_functions:
        if function.getName() not in external_function_names:
            function_decompilation = decompile_function(decompiler, function)
            if function_decompilation:
                decompiled_buffer += function_decompilation

    return decompiled_buffer


def run_decompilation():
    decompiler = DecompInterface()

    # Load up the binary we're decompiling
    decompiler.openProgram(ghidra_app.currentProgram)

    # binary.py will pass a temporary output directory to this script where we can store its output
    output_path = ghidra_app.getScriptArgs()[0]

    # Do the actual decompilation
    decompilation = decompile_program(decompiler)

    # Write the result to a file which binary.py will read
    with open(output_path, "w") as fw:
        fw.write(decompilation)


if __name__ == "__main__":
    run_decompilation()



from janus.language.block import CodeBlock
from janus.language.naive.registry import register_splitter
from janus.language.node import NodeType
from janus.language.splitter import Splitter


@register_splitter("tag")
class TagSplitter(Splitter):
    """
    Splits code by tags inserted into code
    """

    def __init__(self, tag: str, *args, **kwargs):
        kwargs.update(protected_node_types=("chunk",))
        super().__init__(*args, **kwargs)
        self._tag = f"\n{tag}\n"

    def _get_ast(self, code: str) -> CodeBlock:
        chunks = code.split(self._tag)
        children = []
        start_line = 0
        start_byte = 0
        for i, chunk in enumerate(chunks):
            prefix = suffix = self._tag
            if i == 0:
                prefix = ""
            if i == len(chunks) - 1:
                suffix = ""
            end_byte = start_byte + len(bytes(chunk, "utf-8"))
            end_line = start_line + chunk.count("\n")
            end_char = len(chunk) - chunk.rfind("\n") - 1
            node = CodeBlock(
                text=chunk,
                name=f"Chunk {i}",
                id=f"Chunk {i}",
                start_point=(start_line, 0),
                end_point=(end_line, end_char),
                start_byte=start_byte,
                end_byte=end_byte,
                affixes=(prefix, suffix),
                node_type=NodeType("chunk"),
                children=[],
                language=self.language,
                tokens=self._count_tokens(chunk),
            )
            children.append(node)
            start_line = end_line
            start_byte = end_byte
        return CodeBlock(
            text=code,
            name="root",
            id="root",
            start_point=(0, 0),
            end_point=(code.count("\n"), 0),
            start_byte=0,
            end_byte=len(bytes(code, "utf-8")),
            node_type=NodeType("program"),
            children=children,
            language=self.language,
            tokens=self._count_tokens(code),
        )



import re

from langchain.schema.output_parser import BaseOutputParser
from langchain_core.exceptions import OutputParserException
from langchain_core.messages import BaseMessage
from langchain_core.output_parsers import StrOutputParser

from janus.language.block import CodeBlock
from janus.utils.logger import create_logger

log = create_logger(__name__)


class JanusParser:
    def parse_combined_output(self, text: str) -> str:
        """Parse the output text from the LLM when multiple inputs are combined

        Arguments:
            text: The output text from the LLM

        Returns:
            A parsed version of the text
        """
        if isinstance(text, BaseMessage):
            text = text.content
        return text

    def parse_into_block(self, text: str, block: CodeBlock):
        if isinstance(text, BaseMessage):
            text = text.content
        block.text = text

    def set_reference(self, block: CodeBlock):
        pass


class GenericParser(StrOutputParser, JanusParser):
    def parse(self, text: str) -> str:
        if isinstance(text, BaseMessage):
            return text.content
        return text


class CodeParser(BaseOutputParser[str], JanusParser):
    language: str

    def parse(self, text: str) -> str:
        if isinstance(text, BaseMessage):
            text = text.content
        pattern = rf"```[^\S\r\n]*(?:{self.language}[^\S\r\n]*)?\n?(.*?)\n*```"
        code = re.search(pattern, text, re.DOTALL)
        if code is None:
            raise OutputParserException(
                "Code not find code between triple square brackets"
            )
        return str(code.group(1))

    def get_format_instructions(self) -> str:
        return "Output must contain text contained within triple square brackets (```)"



import unittest

from janus.metrics.reading import _repeat_text, flesch, gunning_fog


class TestReading(unittest.TestCase):
    def setUp(self):
        self.text = "This is a sample text for testing readability metrics"

    def test_repeat_text(self):
        """Test the _repeat_text function."""
        repeated_text = _repeat_text(self.text)
        self.assertIsInstance(repeated_text, str)
        self.assertTrue(len(repeated_text.split()) >= 100)

    def test_flesch(self):
        """Test the Flesch readability score."""
        score = flesch(self.text)
        self.assertAlmostEqual(score, 47.3, places=2)

    def test_gunning_fog(self):
        """Test the Gunning-Fog readability score."""
        score = gunning_fog(self.text)
        self.assertAlmostEqual(score, 8.04, places=2)


if __name__ == "__main__":
    unittest.main()



import click
import nltk
import typer
from rouge import Rouge
from typing_extensions import Annotated

from janus.metrics.metric import metric


@metric(help="ROUGE score")
def rouge(
    target: str,
    reference: str,
    granularity: Annotated[
        str,
        typer.Option(
            "--granularity",
            "-g",
            help=(
                "The granularity of the ROUGE score. `n` refers to "
                "ROUGE-N, `l` refers to ROUGE-L, and `w` refers to ROUGE-W."
            ),
            click_type=click.Choice(["n", "l", "w"]),
        ),
    ] = "n",
    n_gram: Annotated[
        int,
        typer.Option(
            "--n-gram",
            "-n",
            help=("The n-gram overlap calculated for ROUGE-N. Can be an integer."),
        ),
    ] = 2,
    score_type: Annotated[
        str,
        typer.Option(
            "--score",
            "-s",
            help=(
                "Whether to use the F-score, precision, or recall. For example, `f` "
                "refers to the F-score, `p` refers to precision, and `r` refers to "
                "recall."
            ),
            click_type=click.Choice(["f", "p", "r"]),
        ),
    ] = "f",
    **kwargs,
) -> float:
    """Calculate the ROUGE Score.

    Arguments:
        target: The target text.
        reference: The reference text.
        granularity: The granularity of the ROUGE score. `n` refers to ROUGE-N, `l`
            refers to ROUGE-L, and `w` refers to ROUGE-W.
        n_gram: The n-gram overlap calculated for ROUGE-N. Can be an integer.
        score_type: Whether to use the F-score, precision, or recall. For example, `f`
            refers to the F-score, `p` refers to precision, and `r` refers to recall.

    Returns:
        The ROUGE score.
    """
    nltk.download("punkt", quiet=True)

    if granularity.lower() == "n":
        metric_name = "rouge-n"
        metric_name_output = f"rouge-{n_gram}"
        max_n = n_gram
    elif granularity.lower() == "l":
        metric_name = "rouge-l"
        metric_name_output = "rouge-l"
        max_n = 4
    elif granularity.lower() == "w":
        metric_name = "rouge-w"
        metric_name_output = "rouge-w"
        max_n = 4
    else:
        raise ValueError("Invalid granularity. Must be one of `n`, `l`, or `w`.")

    if score_type.lower() not in ["f", "p", "r"]:
        raise ValueError("Invalid score type. Must be one of `f`, `p`, or `r`.")

    evaluator = Rouge(
        metrics=[metric_name],
        max_n=max_n,
        limit_length=False,
        length_limit=1_000,
        length_limit_type="words",
        apply_avg=False,
        apply_best=False,
        alpha=0.5,  # Default F1_score
        weight_factor=1.2,
        stemming=True,
    )
    scores = evaluator.get_scores(target, reference)
    return scores[metric_name_output][0][score_type.lower()][0]



# FILEPATH: /Users/mdoyle/projects/janus/janus/metrics/tests/test_file_pairing.py

import unittest
from pathlib import Path

from janus.metrics.file_pairing import (
    FILE_PAIRING_METHODS,
    pair_by_file,
    pair_by_line,
    pair_by_line_comment,
    register_pairing_method,
)


class TestFilePairing(unittest.TestCase):
    def setUp(self):
        self.src = "Hello\nWorld"
        self.cmp = "Hello\nPython"
        self.state = {
            "token_limit": 100,
            "llm": None,
            "lang": "python",
            "target_file": self.src,
            "cmp_file": self.cmp,
        }

    def test_register_pairing_method(self):
        @register_pairing_method(name="test")
        def test_method(src, cmp, state):
            return [(src, cmp)]

        self.assertIn("test", FILE_PAIRING_METHODS)

    def test_pair_by_file(self):
        expected = [(self.src, self.cmp)]
        result = pair_by_file(self.src, self.cmp)
        self.assertEqual(result, expected)

    def test_pair_by_line(self):
        expected = [("Hello", "Hello"), ("World", "Python")]
        result = pair_by_line(self.src, self.cmp)
        self.assertEqual(result, expected)

    def test_pair_by_line_comment(self):
        # This test assumes that the source and comparison files have comments on the
        # same lines
        # You may need to adjust this test based on your specific use case
        self.target = Path(__file__).parent / "target.py"
        self.reference = Path(__file__).parent / "reference.py"
        kwargs = {
            "token_limit": 100,
            "llm": None,
            "lang": "python",
            "target_file": self.target,
            "reference_file": self.reference,
        }
        expected = [("# Hello\n", "# Hello\n")]
        result = pair_by_line_comment(self.src, self.cmp, **kwargs)
        self.assertEqual(result, expected)



import unittest
from pathlib import Path
from typing import Any, Iterable, List, Optional, Type

import pytest
from langchain.schema import Document
from langchain.schema.embeddings import Embeddings
from langchain.schema.vectorstore import VST, VectorStore

from janus.converter.diagram import DiagramGenerator
from janus.converter.requirements import RequirementsDocumenter
from janus.converter.translate import Translator
from janus.language.block import CodeBlock, TranslatedCodeBlock


class MockCollection(VectorStore):
    """Vector store for testing"""

    def __init__(self):
        self._add_texts_calls = 0

    def add_texts(
        self, texts: Iterable[str], metadatas: Optional[List[dict]] = None, **kwargs: Any
    ) -> List[str]:
        self._add_texts_calls += 1
        return ["id"]

    def similarity_search(self, query: str, k: int = 4, **kwargs: Any) -> List[Document]:
        raise NotImplementedError("similarity_search() not implemented!")

    @classmethod
    def from_texts(
        cls: Type[VST],
        texts: List[str],
        embedding: Embeddings,
        metadatas: Optional[List[dict]] = None,
        **kwargs: Any,
    ) -> VST:
        raise NotImplementedError("from_texts() not implemented!")


class TestTranslator(unittest.TestCase):
    """Tests for the Translator class."""

    def setUp(self):
        """Set up the tests."""
        self.translator = Translator(
            model="gpt-4o-mini",
            source_language="fortran",
            target_language="python",
            target_version="3.10",
            splitter_type="ast-flex",
        )
        self.test_file = Path("janus/language/treesitter/_tests/languages/fortran.f90")
        self.TEST_FILE_EMBEDDING_COUNT = 14

        self.req_translator = RequirementsDocumenter(
            model="gpt-4o-mini",
            source_language="fortran",
            prompt_template="requirements",
        )

    @pytest.mark.translate
    def test_translate(self):
        """Test translate method."""
        # Delete a file if it's already there
        python_file = self.test_file.parent / "python" / f"{self.test_file.stem}.py"
        python_file.unlink(missing_ok=True)
        python_file.parent.rmdir() if python_file.parent.is_dir() else None
        self.translator.translate(self.test_file.parent, self.test_file.parent / "python")
        # Only check the top-most level functionality, since it should be handled by other
        # unit tests anyway
        self.assertTrue(python_file.exists())

    def test_invalid_selections(self) -> None:
        """Tests that settings values for the translator will raise exceptions"""
        self.assertRaises(
            ValueError, self.translator.set_target_language, "gobbledy", "goobledy"
        )
        self.assertRaises(
            ValueError, self.translator.set_source_language, "scribbledy-doop"
        )
        self.translator.set_prompt("pish posh")
        self.assertRaises(ValueError, self.translator._load_parameters)


class TestDiagramGenerator(unittest.TestCase):
    """Tests for the DiagramGenerator class."""

    def setUp(self):
        """Set up the tests."""
        self.diagram_generator = DiagramGenerator(
            model="gpt-4o",
            source_language="fortran",
            diagram_type="Activity",
        )

    def test_init(self):
        """Test __init__ method."""
        self.assertEqual(self.diagram_generator._model_name, "gpt-4o")
        self.assertEqual(self.diagram_generator._source_language, "fortran")
        self.assertEqual(self.diagram_generator._diagram_type, "Activity")

    def test_add_translation(self):
        """Test _add_translation method."""
        block = TranslatedCodeBlock(
            original=CodeBlock(
                id="test",
                name="Test Block",
                node_type="function",
                language="python",
                text="print('Hello, World!')",
                start_point=(0, 0),
                end_point=(1, 0),
                start_byte=0,
                end_byte=1,
                tokens=5,
                children=[],
            ),
            language="python",
        )
        self.diagram_generator._add_translation(block)
        self.assertTrue(block.translated)
        self.assertIsNotNone(block.text)
        self.assertIsNotNone(block.tokens)


@pytest.mark.parametrize(
    "source_language,prompt_template,expected_target_language,expected_target_version,",
    [
        ("python", "document_inline", "python", "3.10"),
        ("fortran", "document", "text", None),
        ("mumps", "requirements", "text", None),
        ("python", "simple", "javascript", "es6"),
    ],
)
def test_language_combinations(
    source_language: str,
    prompt_template: str,
    expected_target_language: str,
    expected_target_version: str,
):
    """Tests that translator target language settings are consistent
    with prompt template expectations.
    """
    translator = Translator(model="gpt-4o")
    translator.set_model("gpt-4o")
    translator.set_source_language(source_language)
    translator.set_target_language(expected_target_language, expected_target_version)
    translator.set_prompt(prompt_template)
    translator._load_parameters()
    assert translator._target_language == expected_target_language  # nosec
    assert translator._target_version == expected_target_version  # nosec
    assert translator._splitter.language == source_language  # nosec
    assert translator._splitter.model.model_name == "gpt-4o"  # nosec
    assert translator._prompt_template_name == prompt_template  # nosec



from langchain_core.prompts import ChatPromptTemplate

from janus.llm.models_info import MODEL_PROMPT_ENGINES


class Refiner:
    def refine(
        self, original_prompt: str, original_output: str, errors: str, **kwargs
    ) -> tuple[ChatPromptTemplate, dict[str, str]]:
        """Creates a new prompt based on feedback from original results

        Arguments:
            original_prompt: original prompt used to produce output
            original_output: origial output of llm
            errors: list of errors detected by parser

        Returns:
            Tuple of new prompt and prompt arguments
        """
        raise NotImplementedError


class BasicRefiner(Refiner):
    def __init__(
        self,
        prompt_name: str,
        model_name: str,
        source_language: str,
    ) -> None:
        """Basic refiner, asks llm to fix output of previous prompt given errors

        Arguments:
            prompt_name: refinement prompt name to use
            model_name: name of llm to use
            source_language: source_langauge to use
        """
        self._prompt_name = prompt_name
        self._model_name = model_name
        self._source_language = source_language

    def refine(
        self, original_prompt: str, original_output: str, errors: str, **kwargs
    ) -> tuple[ChatPromptTemplate, dict[str, str]]:
        """Creates a new prompt based on feedback from original results

        Arguments:
            original_prompt: original prompt used to produce output
            original_output: origial output of llm
            errors: list of errors detected by parser

        Returns:
            Tuple of new prompt and prompt arguments
        """
        prompt_engine = MODEL_PROMPT_ENGINES[self._model_name](
            prompt_template=self._prompt_name,
            source_language=self._source_language,
        )
        prompt_arguments = {
            "ORIGINAL_PROMPT": original_prompt,
            "OUTPUT": original_output,
            "ERRORS": errors,
        }
        return prompt_engine.prompt, prompt_arguments



import unittest

from janus.metrics.rouge_score import rouge


class TestRouge(unittest.TestCase):
    def setUp(self):
        self.target = "This is a test sentence."
        self.reference = "This is a reference sentence."

    def test_rouge_with_granularity_n(self):
        score = rouge(
            self.target, self.reference, granularity="n", n_gram=2, score_type="f"
        )
        self.assertIsInstance(score, float)

    def test_rouge_with_granularity_l(self):
        score = rouge(
            self.target, self.reference, granularity="l", n_gram=2, score_type="f"
        )
        self.assertIsInstance(score, float)

    def test_rouge_with_granularity_w(self):
        score = rouge(
            self.target, self.reference, granularity="w", n_gram=2, score_type="f"
        )
        self.assertIsInstance(score, float)

    def test_rouge_with_invalid_granularity(self):
        with self.assertRaises(ValueError):
            rouge(
                self.target,
                self.reference,
                granularity="invalid",
                n_gram=2,
                score_type="f",
            )

    def test_rouge_with_score_type_f(self):
        score = rouge(
            self.target, self.reference, granularity="n", n_gram=2, score_type="f"
        )
        self.assertIsInstance(score, float)

    def test_rouge_with_score_type_p(self):
        score = rouge(
            self.target, self.reference, granularity="n", n_gram=2, score_type="p"
        )
        self.assertIsInstance(score, float)

    def test_rouge_with_score_type_r(self):
        score = rouge(
            self.target, self.reference, granularity="n", n_gram=2, score_type="r"
        )
        self.assertIsInstance(score, float)

    def test_rouge_with_invalid_score_type(self):
        with self.assertRaises(ValueError):
            rouge(
                self.target,
                self.reference,
                granularity="n",
                n_gram=2,
                score_type="invalid",
            )
